# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.resources",
    marshal="google.ads.googleads.v21",
    manifest={
        "AdGroupAdLabel",
    },
)


class AdGroupAdLabel(proto.Message):
    r"""A relationship between an ad group ad and a label.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the ad group ad label. Ad
            group ad label resource names have the form:
            ``customers/{customer_id}/adGroupAdLabels/{ad_group_id}~{ad_id}~{label_id}``
        ad_group_ad (str):
            Immutable. The ad group ad to which the label
            is attached.

            This field is a member of `oneof`_ ``_ad_group_ad``.
        label (str):
            Immutable. The label assigned to the ad group
            ad.

            This field is a member of `oneof`_ ``_label``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    ad_group_ad: str = proto.Field(
        proto.STRING,
        number=4,
        optional=True,
    )
    label: str = proto.Field(
        proto.STRING,
        number=5,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example dismisses a given recommendation.

To retrieve recommendations for text ads, run get_text_ad_recommendations.py.
"""


import argparse
import sys

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.services.services.recommendation_service import (
    RecommendationServiceClient,
)
from google.ads.googleads.v22.services.types.recommendation_service import (
    DismissRecommendationRequest,
    DismissRecommendationResponse,
)


def main(
    client: GoogleAdsClient, customer_id: str, recommendation_id: str
) -> None:
    recommendation_service: RecommendationServiceClient = client.get_service(
        "RecommendationService"
    )
    request: DismissRecommendationRequest = client.get_type(
        "DismissRecommendationRequest"
    )
    operation = request.DismissRecommendationOperation()
    operation.resource_name = recommendation_service.recommendation_path(
        customer_id, recommendation_id
    )
    request.customer_id = customer_id
    request.operations.append(operation)

    response: DismissRecommendationResponse = (
        recommendation_service.dismiss_recommendation(request=request)
    )

    print(
        "Dismissed recommendation with resource name: "
        f"'{response.results[0].resource_name}'."
    )


if __name__ == "__main__":
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description=("Dismisses a recommendation with the given ID.")
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-r",
        "--recommendation_id",
        type=str,
        required=True,
        help="The recommendation ID.",
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(googleads_client, args.customer_id, args.recommendation_id)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.errors",
    marshal="google.ads.googleads.v21",
    manifest={
        "ExtensionFeedItemErrorEnum",
    },
)


class ExtensionFeedItemErrorEnum(proto.Message):
    r"""Container for enum describing possible extension feed item
    error.

    """

    class ExtensionFeedItemError(proto.Enum):
        r"""Enum describing possible extension feed item errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            VALUE_OUT_OF_RANGE (2):
                Value is not within the accepted range.
            URL_LIST_TOO_LONG (3):
                Url list is too long.
            CANNOT_HAVE_RESTRICTION_ON_EMPTY_GEO_TARGETING (4):
                Cannot have a geo targeting restriction
                without having geo targeting.
            CANNOT_SET_WITH_FINAL_URLS (5):
                Cannot simultaneously set sitelink field with
                final urls.
            CANNOT_SET_WITHOUT_FINAL_URLS (6):
                Must set field with final urls.
            INVALID_PHONE_NUMBER (7):
                Phone number for a call extension is invalid.
            PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY (8):
                Phone number for a call extension is not
                supported for the given country code.
            CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED (9):
                A carrier specific number in short format is
                not allowed for call extensions.
            PREMIUM_RATE_NUMBER_NOT_ALLOWED (10):
                Premium rate numbers are not allowed for call
                extensions.
            DISALLOWED_NUMBER_TYPE (11):
                Phone number type for a call extension is not
                allowed. For example, personal number is not
                allowed for a call extension in most regions.
            INVALID_DOMESTIC_PHONE_NUMBER_FORMAT (12):
                Phone number for a call extension does not
                meet domestic format requirements.
            VANITY_PHONE_NUMBER_NOT_ALLOWED (13):
                Vanity phone numbers (for example, those
                including letters) are not allowed for call
                extensions.
            INVALID_CALL_CONVERSION_ACTION (14):
                Call conversion action provided for a call
                extension is invalid.
            CUSTOMER_NOT_ON_ALLOWLIST_FOR_CALLTRACKING (47):
                For a call extension, the customer is not on
                the allow-list for call tracking.
            CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY (16):
                Call tracking is not supported for the given
                country for a call extension.
            CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED (17):
                Customer hasn't consented for call recording,
                which is required for creating/updating call
                feed items. See
                https://support.google.com/google-ads/answer/7412639.
            INVALID_APP_ID (18):
                App id provided for an app extension is
                invalid.
            QUOTES_IN_REVIEW_EXTENSION_SNIPPET (19):
                Quotation marks present in the review text
                for a review extension.
            HYPHENS_IN_REVIEW_EXTENSION_SNIPPET (20):
                Hyphen character present in the review text
                for a review extension.
            REVIEW_EXTENSION_SOURCE_INELIGIBLE (21):
                A denylisted review source name or url was
                provided for a review extension.
            SOURCE_NAME_IN_REVIEW_EXTENSION_TEXT (22):
                Review source name should not be found in the
                review text.
            INCONSISTENT_CURRENCY_CODES (23):
                Inconsistent currency codes.
            PRICE_EXTENSION_HAS_DUPLICATED_HEADERS (24):
                Price extension cannot have duplicated
                headers.
            PRICE_ITEM_HAS_DUPLICATED_HEADER_AND_DESCRIPTION (25):
                Price item cannot have duplicated header and
                description.
            PRICE_EXTENSION_HAS_TOO_FEW_ITEMS (26):
                Price extension has too few items.
            PRICE_EXTENSION_HAS_TOO_MANY_ITEMS (27):
                Price extension has too many items.
            UNSUPPORTED_VALUE (28):
                The input value is not currently supported.
            UNSUPPORTED_VALUE_IN_SELECTED_LANGUAGE (29):
                The input value is not currently supported in
                the selected language of an extension.
            INVALID_DEVICE_PREFERENCE (30):
                Unknown or unsupported device preference.
            INVALID_SCHEDULE_END (31):
                Invalid feed item schedule end time (for
                example, endHour = 24 and endMinute != 0).
            DATE_TIME_MUST_BE_IN_ACCOUNT_TIME_ZONE (32):
                Date time zone does not match the account's
                time zone.
            INVALID_SNIPPETS_HEADER (33):
                Invalid structured snippet header.
            CANNOT_OPERATE_ON_REMOVED_FEED_ITEM (34):
                Cannot operate on removed feed item.
            PHONE_NUMBER_NOT_SUPPORTED_WITH_CALLTRACKING_FOR_COUNTRY (35):
                Phone number not supported when call tracking
                enabled for country.
            CONFLICTING_CALL_CONVERSION_SETTINGS (36):
                Cannot set call_conversion_action while
                call_conversion_tracking_enabled is set to true.
            EXTENSION_TYPE_MISMATCH (37):
                The type of the input extension feed item
                doesn't match the existing extension feed item.
            EXTENSION_SUBTYPE_REQUIRED (38):
                The oneof field extension for example,
                subtype of extension feed item is required.
            EXTENSION_TYPE_UNSUPPORTED (39):
                The referenced feed item is not mapped to a
                supported extension type.
            CANNOT_OPERATE_ON_FEED_WITH_MULTIPLE_MAPPINGS (40):
                Cannot operate on a Feed with more than one
                active FeedMapping.
            CANNOT_OPERATE_ON_FEED_WITH_KEY_ATTRIBUTES (41):
                Cannot operate on a Feed that has key
                attributes.
            INVALID_PRICE_FORMAT (42):
                Input price is not in a valid format.
            PROMOTION_INVALID_TIME (43):
                The promotion time is invalid.
            TOO_MANY_DECIMAL_PLACES_SPECIFIED (44):
                This field has too many decimal places
                specified.
            CONCRETE_EXTENSION_TYPE_REQUIRED (45):
                Concrete sub type of ExtensionFeedItem is
                required for this operation.
            SCHEDULE_END_NOT_AFTER_START (46):
                Feed item schedule end time must be after
                start time.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        VALUE_OUT_OF_RANGE = 2
        URL_LIST_TOO_LONG = 3
        CANNOT_HAVE_RESTRICTION_ON_EMPTY_GEO_TARGETING = 4
        CANNOT_SET_WITH_FINAL_URLS = 5
        CANNOT_SET_WITHOUT_FINAL_URLS = 6
        INVALID_PHONE_NUMBER = 7
        PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY = 8
        CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED = 9
        PREMIUM_RATE_NUMBER_NOT_ALLOWED = 10
        DISALLOWED_NUMBER_TYPE = 11
        INVALID_DOMESTIC_PHONE_NUMBER_FORMAT = 12
        VANITY_PHONE_NUMBER_NOT_ALLOWED = 13
        INVALID_CALL_CONVERSION_ACTION = 14
        CUSTOMER_NOT_ON_ALLOWLIST_FOR_CALLTRACKING = 47
        CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY = 16
        CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED = 17
        INVALID_APP_ID = 18
        QUOTES_IN_REVIEW_EXTENSION_SNIPPET = 19
        HYPHENS_IN_REVIEW_EXTENSION_SNIPPET = 20
        REVIEW_EXTENSION_SOURCE_INELIGIBLE = 21
        SOURCE_NAME_IN_REVIEW_EXTENSION_TEXT = 22
        INCONSISTENT_CURRENCY_CODES = 23
        PRICE_EXTENSION_HAS_DUPLICATED_HEADERS = 24
        PRICE_ITEM_HAS_DUPLICATED_HEADER_AND_DESCRIPTION = 25
        PRICE_EXTENSION_HAS_TOO_FEW_ITEMS = 26
        PRICE_EXTENSION_HAS_TOO_MANY_ITEMS = 27
        UNSUPPORTED_VALUE = 28
        UNSUPPORTED_VALUE_IN_SELECTED_LANGUAGE = 29
        INVALID_DEVICE_PREFERENCE = 30
        INVALID_SCHEDULE_END = 31
        DATE_TIME_MUST_BE_IN_ACCOUNT_TIME_ZONE = 32
        INVALID_SNIPPETS_HEADER = 33
        CANNOT_OPERATE_ON_REMOVED_FEED_ITEM = 34
        PHONE_NUMBER_NOT_SUPPORTED_WITH_CALLTRACKING_FOR_COUNTRY = 35
        CONFLICTING_CALL_CONVERSION_SETTINGS = 36
        EXTENSION_TYPE_MISMATCH = 37
        EXTENSION_SUBTYPE_REQUIRED = 38
        EXTENSION_TYPE_UNSUPPORTED = 39
        CANNOT_OPERATE_ON_FEED_WITH_MULTIPLE_MAPPINGS = 40
        CANNOT_OPERATE_ON_FEED_WITH_KEY_ATTRIBUTES = 41
        INVALID_PRICE_FORMAT = 42
        PROMOTION_INVALID_TIME = 43
        TOO_MANY_DECIMAL_PLACES_SPECIFIED = 44
        CONCRETE_EXTENSION_TYPE_REQUIRED = 45
        SCHEDULE_END_NOT_AFTER_START = 46


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v20.enums.types import (
    response_content_type as gage_response_content_type,
)
from google.ads.googleads.v20.resources.types import (
    conversion_goal_campaign_config as gagr_conversion_goal_campaign_config,
)
from google.protobuf import field_mask_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.services",
    marshal="google.ads.googleads.v20",
    manifest={
        "MutateConversionGoalCampaignConfigsRequest",
        "ConversionGoalCampaignConfigOperation",
        "MutateConversionGoalCampaignConfigsResponse",
        "MutateConversionGoalCampaignConfigResult",
    },
)


class MutateConversionGoalCampaignConfigsRequest(proto.Message):
    r"""Request message for
    [ConversionGoalCampaignConfigService.MutateConversionGoalCampaignConfigs][google.ads.googleads.v20.services.ConversionGoalCampaignConfigService.MutateConversionGoalCampaignConfigs].

    Attributes:
        customer_id (str):
            Required. The ID of the customer whose custom
            conversion goals are being modified.
        operations (MutableSequence[google.ads.googleads.v20.services.types.ConversionGoalCampaignConfigOperation]):
            Required. The list of operations to perform
            on individual conversion goal campaign config.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
        response_content_type (google.ads.googleads.v20.enums.types.ResponseContentTypeEnum.ResponseContentType):
            The response content type setting. Determines
            whether the mutable resource or just the
            resource name should be returned post mutation.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operations: MutableSequence["ConversionGoalCampaignConfigOperation"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="ConversionGoalCampaignConfigOperation",
        )
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=3,
    )
    response_content_type: (
        gage_response_content_type.ResponseContentTypeEnum.ResponseContentType
    ) = proto.Field(
        proto.ENUM,
        number=4,
        enum=gage_response_content_type.ResponseContentTypeEnum.ResponseContentType,
    )


class ConversionGoalCampaignConfigOperation(proto.Message):
    r"""A single operation (update) on a conversion goal campaign
    config.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            FieldMask that determines which resource
            fields are modified in an update.
        update (google.ads.googleads.v20.resources.types.ConversionGoalCampaignConfig):
            Update operation: The conversion goal
            campaign config is expected to have a valid
            resource name.

            This field is a member of `oneof`_ ``operation``.
    """

    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )
    update: (
        gagr_conversion_goal_campaign_config.ConversionGoalCampaignConfig
    ) = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="operation",
        message=gagr_conversion_goal_campaign_config.ConversionGoalCampaignConfig,
    )


class MutateConversionGoalCampaignConfigsResponse(proto.Message):
    r"""Response message for a conversion goal campaign config
    mutate.

    Attributes:
        results (MutableSequence[google.ads.googleads.v20.services.types.MutateConversionGoalCampaignConfigResult]):
            All results for the mutate.
    """

    results: MutableSequence["MutateConversionGoalCampaignConfigResult"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="MutateConversionGoalCampaignConfigResult",
        )
    )


class MutateConversionGoalCampaignConfigResult(proto.Message):
    r"""The result for the conversion goal campaign config mutate.

    Attributes:
        resource_name (str):
            Returned for successful operations.
        conversion_goal_campaign_config (google.ads.googleads.v20.resources.types.ConversionGoalCampaignConfig):
            The mutated ConversionGoalCampaignConfig with only mutable
            fields after mutate. The field will only be returned when
            response_content_type is set to "MUTABLE_RESOURCE".
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    conversion_goal_campaign_config: (
        gagr_conversion_goal_campaign_config.ConversionGoalCampaignConfig
    ) = proto.Field(
        proto.MESSAGE,
        number=2,
        message=gagr_conversion_goal_campaign_config.ConversionGoalCampaignConfig,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.enums.types import asset_field_type
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.services",
    marshal="google.ads.googleads.v22",
    manifest={
        "RemoveCampaignAutomaticallyCreatedAssetRequest",
        "RemoveCampaignAutomaticallyCreatedAssetOperation",
        "RemoveCampaignAutomaticallyCreatedAssetResponse",
    },
)


class RemoveCampaignAutomaticallyCreatedAssetRequest(proto.Message):
    r"""Request message for
    [AutomaticallyCreatedAssetRemovalService.RemoveCampaignAutomaticallyCreatedAsset][google.ads.googleads.v22.services.AutomaticallyCreatedAssetRemovalService.RemoveCampaignAutomaticallyCreatedAsset].

    Attributes:
        customer_id (str):
            Required. The ID of the customer whose assets
            are being removed.
        operations (MutableSequence[google.ads.googleads.v22.services.types.RemoveCampaignAutomaticallyCreatedAssetOperation]):
            Required. The list of operations.
        partial_failure (bool):
            Required. If true, successful operations will
            be carried out and invalid operations will
            return errors. If false, all operations will be
            carried out in one transaction if and only if
            they are all valid.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operations: MutableSequence[
        "RemoveCampaignAutomaticallyCreatedAssetOperation"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="RemoveCampaignAutomaticallyCreatedAssetOperation",
    )
    partial_failure: bool = proto.Field(
        proto.BOOL,
        number=3,
    )


class RemoveCampaignAutomaticallyCreatedAssetOperation(proto.Message):
    r"""A single operation to remove an automatically created asset
    from a campaign.

    Attributes:
        campaign (str):
            Required. The resource name of the campaign.
        asset (str):
            Required. The resource name of the asset to
            remove.
        field_type (google.ads.googleads.v22.enums.types.AssetFieldTypeEnum.AssetFieldType):
            Required. The field type of the asset to
            remove.
    """

    campaign: str = proto.Field(
        proto.STRING,
        number=1,
    )
    asset: str = proto.Field(
        proto.STRING,
        number=2,
    )
    field_type: asset_field_type.AssetFieldTypeEnum.AssetFieldType = (
        proto.Field(
            proto.ENUM,
            number=3,
            enum=asset_field_type.AssetFieldTypeEnum.AssetFieldType,
        )
    )


class RemoveCampaignAutomaticallyCreatedAssetResponse(proto.Message):
    r"""Response message for
    [AutomaticallyCreatedAssetRemovalService.RemoveCampaignAutomaticallyCreatedAsset][google.ads.googleads.v22.services.AutomaticallyCreatedAssetRemovalService.RemoveCampaignAutomaticallyCreatedAsset].

    Attributes:
        partial_failure_error (google.rpc.status_pb2.Status):
            Errors that pertain to
            AutomaticallyCreatedAssetRemoval failures in the
            partial failure mode. Returned when all errors
            occur inside the operations. If any errors occur
            outside the operations (for example, auth
            errors), RPC level error will be returned. See
            https://developers.google.com/google-ads/api/docs/best-practices/partial-failures
            for more information about partial failure.
    """

    partial_failure_error: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=1,
        message=status_pb2.Status,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v20.enums.types import custom_conversion_goal_status


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.resources",
    marshal="google.ads.googleads.v20",
    manifest={
        "CustomConversionGoal",
    },
)


class CustomConversionGoal(proto.Message):
    r"""Custom conversion goal that can make arbitrary conversion
    actions biddable.

    Attributes:
        resource_name (str):
            Immutable. The resource name of the custom conversion goal.
            Custom conversion goal resource names have the form:

            ``customers/{customer_id}/customConversionGoals/{goal_id}``
        id (int):
            Immutable. The ID for this custom conversion
            goal.
        name (str):
            The name for this custom conversion goal.
        conversion_actions (MutableSequence[str]):
            Conversion actions that the custom conversion
            goal makes biddable.
        status (google.ads.googleads.v20.enums.types.CustomConversionGoalStatusEnum.CustomConversionGoalStatus):
            The status of the custom conversion goal.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=2,
    )
    name: str = proto.Field(
        proto.STRING,
        number=3,
    )
    conversion_actions: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=4,
    )
    status: (
        custom_conversion_goal_status.CustomConversionGoalStatusEnum.CustomConversionGoalStatus
    ) = proto.Field(
        proto.ENUM,
        number=5,
        enum=custom_conversion_goal_status.CustomConversionGoalStatusEnum.CustomConversionGoalStatus,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from google.ads.googleads.v20 import gapic_version as package_version

__version__ = package_version.__version__


from .types.access_invitation_status import AccessInvitationStatusEnum
from .types.access_reason import AccessReasonEnum
from .types.access_role import AccessRoleEnum
from .types.account_budget_proposal_status import (
    AccountBudgetProposalStatusEnum,
)
from .types.account_budget_proposal_type import AccountBudgetProposalTypeEnum
from .types.account_budget_status import AccountBudgetStatusEnum
from .types.account_link_status import AccountLinkStatusEnum
from .types.ad_destination_type import AdDestinationTypeEnum
from .types.ad_format_type import AdFormatTypeEnum
from .types.ad_group_ad_primary_status import AdGroupAdPrimaryStatusEnum
from .types.ad_group_ad_primary_status_reason import (
    AdGroupAdPrimaryStatusReasonEnum,
)
from .types.ad_group_ad_rotation_mode import AdGroupAdRotationModeEnum
from .types.ad_group_ad_status import AdGroupAdStatusEnum
from .types.ad_group_criterion_approval_status import (
    AdGroupCriterionApprovalStatusEnum,
)
from .types.ad_group_criterion_primary_status import (
    AdGroupCriterionPrimaryStatusEnum,
)
from .types.ad_group_criterion_primary_status_reason import (
    AdGroupCriterionPrimaryStatusReasonEnum,
)
from .types.ad_group_criterion_status import AdGroupCriterionStatusEnum
from .types.ad_group_primary_status import AdGroupPrimaryStatusEnum
from .types.ad_group_primary_status_reason import AdGroupPrimaryStatusReasonEnum
from .types.ad_group_status import AdGroupStatusEnum
from .types.ad_group_type import AdGroupTypeEnum
from .types.ad_network_type import AdNetworkTypeEnum
from .types.ad_serving_optimization_status import (
    AdServingOptimizationStatusEnum,
)
from .types.ad_strength import AdStrengthEnum
from .types.ad_strength_action_item_type import AdStrengthActionItemTypeEnum
from .types.ad_type import AdTypeEnum
from .types.advertising_channel_sub_type import AdvertisingChannelSubTypeEnum
from .types.advertising_channel_type import AdvertisingChannelTypeEnum
from .types.age_range_type import AgeRangeTypeEnum
from .types.android_privacy_interaction_type import (
    AndroidPrivacyInteractionTypeEnum,
)
from .types.android_privacy_network_type import AndroidPrivacyNetworkTypeEnum
from .types.app_bidding_goal import AppBiddingGoalEnum
from .types.app_campaign_app_store import AppCampaignAppStoreEnum
from .types.app_campaign_bidding_strategy_goal_type import (
    AppCampaignBiddingStrategyGoalTypeEnum,
)
from .types.app_payment_model_type import AppPaymentModelTypeEnum
from .types.app_url_operating_system_type import AppUrlOperatingSystemTypeEnum
from .types.application_instance import ApplicationInstanceEnum
from .types.asset_automation_status import AssetAutomationStatusEnum
from .types.asset_automation_type import AssetAutomationTypeEnum
from .types.asset_coverage_video_aspect_ratio_requirement import (
    AssetCoverageVideoAspectRatioRequirementEnum,
)
from .types.asset_field_type import AssetFieldTypeEnum
from .types.asset_group_primary_status import AssetGroupPrimaryStatusEnum
from .types.asset_group_primary_status_reason import (
    AssetGroupPrimaryStatusReasonEnum,
)
from .types.asset_group_signal_approval_status import (
    AssetGroupSignalApprovalStatusEnum,
)
from .types.asset_group_status import AssetGroupStatusEnum
from .types.asset_link_primary_status import AssetLinkPrimaryStatusEnum
from .types.asset_link_primary_status_reason import (
    AssetLinkPrimaryStatusReasonEnum,
)
from .types.asset_link_status import AssetLinkStatusEnum
from .types.asset_offline_evaluation_error_reasons import (
    AssetOfflineEvaluationErrorReasonsEnum,
)
from .types.asset_performance_label import AssetPerformanceLabelEnum
from .types.asset_set_asset_status import AssetSetAssetStatusEnum
from .types.asset_set_link_status import AssetSetLinkStatusEnum
from .types.asset_set_status import AssetSetStatusEnum
from .types.asset_set_type import AssetSetTypeEnum
from .types.asset_source import AssetSourceEnum
from .types.asset_type import AssetTypeEnum
from .types.async_action_status import AsyncActionStatusEnum
from .types.attribution_model import AttributionModelEnum
from .types.audience_insights_dimension import AudienceInsightsDimensionEnum
from .types.audience_insights_marketing_objective import (
    AudienceInsightsMarketingObjectiveEnum,
)
from .types.audience_scope import AudienceScopeEnum
from .types.audience_status import AudienceStatusEnum
from .types.batch_job_status import BatchJobStatusEnum
from .types.bid_modifier_source import BidModifierSourceEnum
from .types.bidding_source import BiddingSourceEnum
from .types.bidding_strategy_status import BiddingStrategyStatusEnum
from .types.bidding_strategy_system_status import (
    BiddingStrategySystemStatusEnum,
)
from .types.bidding_strategy_type import BiddingStrategyTypeEnum
from .types.billing_setup_status import BillingSetupStatusEnum
from .types.brand_request_rejection_reason import (
    BrandRequestRejectionReasonEnum,
)
from .types.brand_safety_suitability import BrandSafetySuitabilityEnum
from .types.brand_state import BrandStateEnum
from .types.budget_campaign_association_status import (
    BudgetCampaignAssociationStatusEnum,
)
from .types.budget_delivery_method import BudgetDeliveryMethodEnum
from .types.budget_period import BudgetPeriodEnum
from .types.budget_status import BudgetStatusEnum
from .types.budget_type import BudgetTypeEnum
from .types.business_message_call_to_action_type import (
    BusinessMessageCallToActionTypeEnum,
)
from .types.business_message_provider import BusinessMessageProviderEnum
from .types.call_conversion_reporting_state import (
    CallConversionReportingStateEnum,
)
from .types.call_to_action_type import CallToActionTypeEnum
from .types.call_tracking_display_location import (
    CallTrackingDisplayLocationEnum,
)
from .types.call_type import CallTypeEnum
from .types.campaign_criterion_status import CampaignCriterionStatusEnum
from .types.campaign_draft_status import CampaignDraftStatusEnum
from .types.campaign_experiment_type import CampaignExperimentTypeEnum
from .types.campaign_group_status import CampaignGroupStatusEnum
from .types.campaign_keyword_match_type import CampaignKeywordMatchTypeEnum
from .types.campaign_primary_status import CampaignPrimaryStatusEnum
from .types.campaign_primary_status_reason import (
    CampaignPrimaryStatusReasonEnum,
)
from .types.campaign_serving_status import CampaignServingStatusEnum
from .types.campaign_shared_set_status import CampaignSharedSetStatusEnum
from .types.campaign_status import CampaignStatusEnum
from .types.chain_relationship_type import ChainRelationshipTypeEnum
from .types.change_client_type import ChangeClientTypeEnum
from .types.change_event_resource_type import ChangeEventResourceTypeEnum
from .types.change_status_operation import ChangeStatusOperationEnum
from .types.change_status_resource_type import ChangeStatusResourceTypeEnum
from .types.click_type import ClickTypeEnum
from .types.combined_audience_status import CombinedAudienceStatusEnum
from .types.consent_status import ConsentStatusEnum
from .types.content_label_type import ContentLabelTypeEnum
from .types.conversion_action_category import ConversionActionCategoryEnum
from .types.conversion_action_counting_type import (
    ConversionActionCountingTypeEnum,
)
from .types.conversion_action_status import ConversionActionStatusEnum
from .types.conversion_action_type import ConversionActionTypeEnum
from .types.conversion_adjustment_type import ConversionAdjustmentTypeEnum
from .types.conversion_attribution_event_type import (
    ConversionAttributionEventTypeEnum,
)
from .types.conversion_custom_variable_status import (
    ConversionCustomVariableStatusEnum,
)
from .types.conversion_customer_type import ConversionCustomerTypeEnum
from .types.conversion_environment_enum import ConversionEnvironmentEnum
from .types.conversion_lag_bucket import ConversionLagBucketEnum
from .types.conversion_or_adjustment_lag_bucket import (
    ConversionOrAdjustmentLagBucketEnum,
)
from .types.conversion_origin import ConversionOriginEnum
from .types.conversion_tracking_status_enum import ConversionTrackingStatusEnum
from .types.conversion_value_rule_primary_dimension import (
    ConversionValueRulePrimaryDimensionEnum,
)
from .types.conversion_value_rule_set_status import (
    ConversionValueRuleSetStatusEnum,
)
from .types.conversion_value_rule_status import ConversionValueRuleStatusEnum
from .types.converting_user_prior_engagement_type_and_ltv_bucket import (
    ConvertingUserPriorEngagementTypeAndLtvBucketEnum,
)
from .types.criterion_category_channel_availability_mode import (
    CriterionCategoryChannelAvailabilityModeEnum,
)
from .types.criterion_category_locale_availability_mode import (
    CriterionCategoryLocaleAvailabilityModeEnum,
)
from .types.criterion_system_serving_status import (
    CriterionSystemServingStatusEnum,
)
from .types.criterion_type import CriterionTypeEnum
from .types.custom_audience_member_type import CustomAudienceMemberTypeEnum
from .types.custom_audience_status import CustomAudienceStatusEnum
from .types.custom_audience_type import CustomAudienceTypeEnum
from .types.custom_conversion_goal_status import CustomConversionGoalStatusEnum
from .types.custom_interest_member_type import CustomInterestMemberTypeEnum
from .types.custom_interest_status import CustomInterestStatusEnum
from .types.custom_interest_type import CustomInterestTypeEnum
from .types.customer_acquisition_optimization_mode import (
    CustomerAcquisitionOptimizationModeEnum,
)
from .types.customer_match_upload_key_type import CustomerMatchUploadKeyTypeEnum
from .types.customer_pay_per_conversion_eligibility_failure_reason import (
    CustomerPayPerConversionEligibilityFailureReasonEnum,
)
from .types.customer_status import CustomerStatusEnum
from .types.customizer_attribute_status import CustomizerAttributeStatusEnum
from .types.customizer_attribute_type import CustomizerAttributeTypeEnum
from .types.customizer_value_status import CustomizerValueStatusEnum
from .types.data_driven_model_status import DataDrivenModelStatusEnum
from .types.data_link_status import DataLinkStatusEnum
from .types.data_link_type import DataLinkTypeEnum
from .types.day_of_week import DayOfWeekEnum
from .types.demand_gen_channel_config import DemandGenChannelConfigEnum
from .types.demand_gen_channel_strategy import DemandGenChannelStrategyEnum
from .types.device import DeviceEnum
from .types.display_ad_format_setting import DisplayAdFormatSettingEnum
from .types.display_upload_product_type import DisplayUploadProductTypeEnum
from .types.distance_bucket import DistanceBucketEnum
from .types.eu_political_advertising_status import (
    EuPoliticalAdvertisingStatusEnum,
)
from .types.experiment_metric import ExperimentMetricEnum
from .types.experiment_metric_direction import ExperimentMetricDirectionEnum
from .types.experiment_status import ExperimentStatusEnum
from .types.experiment_type import ExperimentTypeEnum
from .types.external_conversion_source import ExternalConversionSourceEnum
from .types.fixed_cpm_goal import FixedCpmGoalEnum
from .types.fixed_cpm_target_frequency_time_unit import (
    FixedCpmTargetFrequencyTimeUnitEnum,
)
from .types.frequency_cap_event_type import FrequencyCapEventTypeEnum
from .types.frequency_cap_level import FrequencyCapLevelEnum
from .types.frequency_cap_time_unit import FrequencyCapTimeUnitEnum
from .types.gender_type import GenderTypeEnum
from .types.geo_target_constant_status import GeoTargetConstantStatusEnum
from .types.geo_targeting_type import GeoTargetingTypeEnum
from .types.goal_config_level import GoalConfigLevelEnum
from .types.google_ads_field_category import GoogleAdsFieldCategoryEnum
from .types.google_ads_field_data_type import GoogleAdsFieldDataTypeEnum
from .types.google_voice_call_status import GoogleVoiceCallStatusEnum
from .types.hotel_asset_suggestion_status import HotelAssetSuggestionStatusEnum
from .types.hotel_date_selection_type import HotelDateSelectionTypeEnum
from .types.hotel_price_bucket import HotelPriceBucketEnum
from .types.hotel_rate_type import HotelRateTypeEnum
from .types.hotel_reconciliation_status import HotelReconciliationStatusEnum
from .types.identity_verification_program import IdentityVerificationProgramEnum
from .types.identity_verification_program_status import (
    IdentityVerificationProgramStatusEnum,
)
from .types.income_range_type import IncomeRangeTypeEnum
from .types.insights_knowledge_graph_entity_capabilities import (
    InsightsKnowledgeGraphEntityCapabilitiesEnum,
)
from .types.insights_trend import InsightsTrendEnum
from .types.interaction_event_type import InteractionEventTypeEnum
from .types.interaction_type import InteractionTypeEnum
from .types.invoice_type import InvoiceTypeEnum
from .types.keyword_match_type import KeywordMatchTypeEnum
from .types.keyword_plan_aggregate_metric_type import (
    KeywordPlanAggregateMetricTypeEnum,
)
from .types.keyword_plan_competition_level import (
    KeywordPlanCompetitionLevelEnum,
)
from .types.keyword_plan_concept_group_type import (
    KeywordPlanConceptGroupTypeEnum,
)
from .types.keyword_plan_forecast_interval import (
    KeywordPlanForecastIntervalEnum,
)
from .types.keyword_plan_keyword_annotation import (
    KeywordPlanKeywordAnnotationEnum,
)
from .types.keyword_plan_network import KeywordPlanNetworkEnum
from .types.label_status import LabelStatusEnum
from .types.lead_form_call_to_action_type import LeadFormCallToActionTypeEnum
from .types.lead_form_desired_intent import LeadFormDesiredIntentEnum
from .types.lead_form_field_user_input_type import (
    LeadFormFieldUserInputTypeEnum,
)
from .types.lead_form_post_submit_call_to_action_type import (
    LeadFormPostSubmitCallToActionTypeEnum,
)
from .types.legacy_app_install_ad_app_store import (
    LegacyAppInstallAdAppStoreEnum,
)
from .types.linked_account_type import LinkedAccountTypeEnum
from .types.linked_product_type import LinkedProductTypeEnum
from .types.listing_group_filter_custom_attribute_index import (
    ListingGroupFilterCustomAttributeIndexEnum,
)
from .types.listing_group_filter_listing_source import (
    ListingGroupFilterListingSourceEnum,
)
from .types.listing_group_filter_product_category_level import (
    ListingGroupFilterProductCategoryLevelEnum,
)
from .types.listing_group_filter_product_channel import (
    ListingGroupFilterProductChannelEnum,
)
from .types.listing_group_filter_product_condition import (
    ListingGroupFilterProductConditionEnum,
)
from .types.listing_group_filter_product_type_level import (
    ListingGroupFilterProductTypeLevelEnum,
)
from .types.listing_group_filter_type_enum import ListingGroupFilterTypeEnum
from .types.listing_group_type import ListingGroupTypeEnum
from .types.listing_type import ListingTypeEnum
from .types.local_services_business_registration_check_rejection_reason import (
    LocalServicesBusinessRegistrationCheckRejectionReasonEnum,
)
from .types.local_services_business_registration_type import (
    LocalServicesBusinessRegistrationTypeEnum,
)
from .types.local_services_conversation_type import (
    LocalServicesLeadConversationTypeEnum,
)
from .types.local_services_employee_status import (
    LocalServicesEmployeeStatusEnum,
)
from .types.local_services_employee_type import LocalServicesEmployeeTypeEnum
from .types.local_services_insurance_rejection_reason import (
    LocalServicesInsuranceRejectionReasonEnum,
)
from .types.local_services_lead_credit_issuance_decision import (
    LocalServicesLeadCreditIssuanceDecisionEnum,
)
from .types.local_services_lead_credit_state import LocalServicesCreditStateEnum
from .types.local_services_lead_status import LocalServicesLeadStatusEnum
from .types.local_services_lead_survey_answer import (
    LocalServicesLeadSurveyAnswerEnum,
)
from .types.local_services_lead_survey_dissatisfied_reason import (
    LocalServicesLeadSurveyDissatisfiedReasonEnum,
)
from .types.local_services_lead_survey_satisfied_reason import (
    LocalServicesLeadSurveySatisfiedReasonEnum,
)
from .types.local_services_lead_type import LocalServicesLeadTypeEnum
from .types.local_services_license_rejection_reason import (
    LocalServicesLicenseRejectionReasonEnum,
)
from .types.local_services_participant_type import (
    LocalServicesParticipantTypeEnum,
)
from .types.local_services_verification_artifact_status import (
    LocalServicesVerificationArtifactStatusEnum,
)
from .types.local_services_verification_artifact_type import (
    LocalServicesVerificationArtifactTypeEnum,
)
from .types.local_services_verification_status import (
    LocalServicesVerificationStatusEnum,
)
from .types.location_group_radius_units import LocationGroupRadiusUnitsEnum
from .types.location_ownership_type import LocationOwnershipTypeEnum
from .types.location_source_type import LocationSourceTypeEnum
from .types.location_string_filter_type import LocationStringFilterTypeEnum
from .types.lookalike_expansion_level import LookalikeExpansionLevelEnum
from .types.manager_link_status import ManagerLinkStatusEnum
from .types.media_type import MediaTypeEnum
from .types.mime_type import MimeTypeEnum
from .types.minute_of_hour import MinuteOfHourEnum
from .types.mobile_app_vendor import MobileAppVendorEnum
from .types.mobile_device_type import MobileDeviceTypeEnum
from .types.month_of_year import MonthOfYearEnum
from .types.negative_geo_target_type import NegativeGeoTargetTypeEnum
from .types.non_skippable_max_duration import NonSkippableMaxDurationEnum
from .types.non_skippable_min_duration import NonSkippableMinDurationEnum
from .types.offline_conversion_diagnostic_status_enum import (
    OfflineConversionDiagnosticStatusEnum,
)
from .types.offline_event_upload_client_enum import OfflineEventUploadClientEnum
from .types.offline_user_data_job_failure_reason import (
    OfflineUserDataJobFailureReasonEnum,
)
from .types.offline_user_data_job_match_rate_range import (
    OfflineUserDataJobMatchRateRangeEnum,
)
from .types.offline_user_data_job_status import OfflineUserDataJobStatusEnum
from .types.offline_user_data_job_type import OfflineUserDataJobTypeEnum
from .types.operating_system_version_operator_type import (
    OperatingSystemVersionOperatorTypeEnum,
)
from .types.optimization_goal_type import OptimizationGoalTypeEnum
from .types.parental_status_type import ParentalStatusTypeEnum
from .types.payment_mode import PaymentModeEnum
from .types.performance_max_upgrade_status import (
    PerformanceMaxUpgradeStatusEnum,
)
from .types.placement_type import PlacementTypeEnum
from .types.policy_approval_status import PolicyApprovalStatusEnum
from .types.policy_review_status import PolicyReviewStatusEnum
from .types.policy_topic_entry_type import PolicyTopicEntryTypeEnum
from .types.policy_topic_evidence_destination_mismatch_url_type import (
    PolicyTopicEvidenceDestinationMismatchUrlTypeEnum,
)
from .types.policy_topic_evidence_destination_not_working_device import (
    PolicyTopicEvidenceDestinationNotWorkingDeviceEnum,
)
from .types.policy_topic_evidence_destination_not_working_dns_error_type import (
    PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum,
)
from .types.positive_geo_target_type import PositiveGeoTargetTypeEnum
from .types.price_extension_price_qualifier import (
    PriceExtensionPriceQualifierEnum,
)
from .types.price_extension_price_unit import PriceExtensionPriceUnitEnum
from .types.price_extension_type import PriceExtensionTypeEnum
from .types.product_availability import ProductAvailabilityEnum
from .types.product_category_level import ProductCategoryLevelEnum
from .types.product_category_state import ProductCategoryStateEnum
from .types.product_channel import ProductChannelEnum
from .types.product_channel_exclusivity import ProductChannelExclusivityEnum
from .types.product_condition import ProductConditionEnum
from .types.product_custom_attribute_index import (
    ProductCustomAttributeIndexEnum,
)
from .types.product_issue_severity import ProductIssueSeverityEnum
from .types.product_link_invitation_status import (
    ProductLinkInvitationStatusEnum,
)
from .types.product_status import ProductStatusEnum
from .types.product_type_level import ProductTypeLevelEnum
from .types.promotion_extension_discount_modifier import (
    PromotionExtensionDiscountModifierEnum,
)
from .types.promotion_extension_occasion import PromotionExtensionOccasionEnum
from .types.proximity_radius_units import ProximityRadiusUnitsEnum
from .types.quality_score_bucket import QualityScoreBucketEnum
from .types.reach_plan_age_range import ReachPlanAgeRangeEnum
from .types.reach_plan_conversion_rate_model import (
    ReachPlanConversionRateModelEnum,
)
from .types.reach_plan_network import ReachPlanNetworkEnum
from .types.reach_plan_plannable_user_list_status import (
    ReachPlanPlannableUserListStatusEnum,
)
from .types.reach_plan_surface import ReachPlanSurfaceEnum
from .types.recommendation_subscription_status import (
    RecommendationSubscriptionStatusEnum,
)
from .types.recommendation_type import RecommendationTypeEnum
from .types.resource_change_operation import ResourceChangeOperationEnum
from .types.resource_limit_type import ResourceLimitTypeEnum
from .types.response_content_type import ResponseContentTypeEnum
from .types.search_engine_results_page_type import (
    SearchEngineResultsPageTypeEnum,
)
from .types.search_term_match_type import SearchTermMatchTypeEnum
from .types.search_term_targeting_status import SearchTermTargetingStatusEnum
from .types.seasonality_event_scope import SeasonalityEventScopeEnum
from .types.seasonality_event_status import SeasonalityEventStatusEnum
from .types.served_asset_field_type import ServedAssetFieldTypeEnum
from .types.shared_set_status import SharedSetStatusEnum
from .types.shared_set_type import SharedSetTypeEnum
from .types.shopping_add_products_to_campaign_recommendation_enum import (
    ShoppingAddProductsToCampaignRecommendationEnum,
)
from .types.simulation_modification_method import (
    SimulationModificationMethodEnum,
)
from .types.simulation_type import SimulationTypeEnum
from .types.sk_ad_network_ad_event_type import SkAdNetworkAdEventTypeEnum
from .types.sk_ad_network_attribution_credit import (
    SkAdNetworkAttributionCreditEnum,
)
from .types.sk_ad_network_coarse_conversion_value import (
    SkAdNetworkCoarseConversionValueEnum,
)
from .types.sk_ad_network_source_type import SkAdNetworkSourceTypeEnum
from .types.sk_ad_network_user_type import SkAdNetworkUserTypeEnum
from .types.slot import SlotEnum
from .types.smart_campaign_not_eligible_reason import (
    SmartCampaignNotEligibleReasonEnum,
)
from .types.smart_campaign_status import SmartCampaignStatusEnum
from .types.spending_limit_type import SpendingLimitTypeEnum
from .types.summary_row_setting import SummaryRowSettingEnum
from .types.system_managed_entity_source import SystemManagedResourceSourceEnum
from .types.target_cpa_opt_in_recommendation_goal import (
    TargetCpaOptInRecommendationGoalEnum,
)
from .types.target_frequency_time_unit import TargetFrequencyTimeUnitEnum
from .types.target_impression_share_location import (
    TargetImpressionShareLocationEnum,
)
from .types.targeting_dimension import TargetingDimensionEnum
from .types.time_type import TimeTypeEnum
from .types.tracking_code_page_format import TrackingCodePageFormatEnum
from .types.tracking_code_type import TrackingCodeTypeEnum
from .types.user_identifier_source import UserIdentifierSourceEnum
from .types.user_interest_taxonomy_type import UserInterestTaxonomyTypeEnum
from .types.user_list_access_status import UserListAccessStatusEnum
from .types.user_list_closing_reason import UserListClosingReasonEnum
from .types.user_list_crm_data_source_type import UserListCrmDataSourceTypeEnum
from .types.user_list_customer_type_category import (
    UserListCustomerTypeCategoryEnum,
)
from .types.user_list_date_rule_item_operator import (
    UserListDateRuleItemOperatorEnum,
)
from .types.user_list_flexible_rule_operator import (
    UserListFlexibleRuleOperatorEnum,
)
from .types.user_list_logical_rule_operator import (
    UserListLogicalRuleOperatorEnum,
)
from .types.user_list_membership_status import UserListMembershipStatusEnum
from .types.user_list_number_rule_item_operator import (
    UserListNumberRuleItemOperatorEnum,
)
from .types.user_list_prepopulation_status import (
    UserListPrepopulationStatusEnum,
)
from .types.user_list_rule_type import UserListRuleTypeEnum
from .types.user_list_size_range import UserListSizeRangeEnum
from .types.user_list_string_rule_item_operator import (
    UserListStringRuleItemOperatorEnum,
)
from .types.user_list_type import UserListTypeEnum
from .types.value_rule_device_type import ValueRuleDeviceTypeEnum
from .types.value_rule_geo_location_match_type import (
    ValueRuleGeoLocationMatchTypeEnum,
)
from .types.value_rule_operation import ValueRuleOperationEnum
from .types.value_rule_set_attachment_type import ValueRuleSetAttachmentTypeEnum
from .types.value_rule_set_dimension import ValueRuleSetDimensionEnum
from .types.vanity_pharma_display_url_mode import VanityPharmaDisplayUrlModeEnum
from .types.vanity_pharma_text import VanityPharmaTextEnum
from .types.video_ad_format_restriction import VideoAdFormatRestrictionEnum
from .types.video_thumbnail import VideoThumbnailEnum
from .types.webpage_condition_operand import WebpageConditionOperandEnum
from .types.webpage_condition_operator import WebpageConditionOperatorEnum

__all__ = (
    "AccessInvitationStatusEnum",
    "AccessReasonEnum",
    "AccessRoleEnum",
    "AccountBudgetProposalStatusEnum",
    "AccountBudgetProposalTypeEnum",
    "AccountBudgetStatusEnum",
    "AccountLinkStatusEnum",
    "AdDestinationTypeEnum",
    "AdFormatTypeEnum",
    "AdGroupAdPrimaryStatusEnum",
    "AdGroupAdPrimaryStatusReasonEnum",
    "AdGroupAdRotationModeEnum",
    "AdGroupAdStatusEnum",
    "AdGroupCriterionApprovalStatusEnum",
    "AdGroupCriterionPrimaryStatusEnum",
    "AdGroupCriterionPrimaryStatusReasonEnum",
    "AdGroupCriterionStatusEnum",
    "AdGroupPrimaryStatusEnum",
    "AdGroupPrimaryStatusReasonEnum",
    "AdGroupStatusEnum",
    "AdGroupTypeEnum",
    "AdNetworkTypeEnum",
    "AdServingOptimizationStatusEnum",
    "AdStrengthActionItemTypeEnum",
    "AdStrengthEnum",
    "AdTypeEnum",
    "AdvertisingChannelSubTypeEnum",
    "AdvertisingChannelTypeEnum",
    "AgeRangeTypeEnum",
    "AndroidPrivacyInteractionTypeEnum",
    "AndroidPrivacyNetworkTypeEnum",
    "AppBiddingGoalEnum",
    "AppCampaignAppStoreEnum",
    "AppCampaignBiddingStrategyGoalTypeEnum",
    "AppPaymentModelTypeEnum",
    "AppUrlOperatingSystemTypeEnum",
    "ApplicationInstanceEnum",
    "AssetAutomationStatusEnum",
    "AssetAutomationTypeEnum",
    "AssetCoverageVideoAspectRatioRequirementEnum",
    "AssetFieldTypeEnum",
    "AssetGroupPrimaryStatusEnum",
    "AssetGroupPrimaryStatusReasonEnum",
    "AssetGroupSignalApprovalStatusEnum",
    "AssetGroupStatusEnum",
    "AssetLinkPrimaryStatusEnum",
    "AssetLinkPrimaryStatusReasonEnum",
    "AssetLinkStatusEnum",
    "AssetOfflineEvaluationErrorReasonsEnum",
    "AssetPerformanceLabelEnum",
    "AssetSetAssetStatusEnum",
    "AssetSetLinkStatusEnum",
    "AssetSetStatusEnum",
    "AssetSetTypeEnum",
    "AssetSourceEnum",
    "AssetTypeEnum",
    "AsyncActionStatusEnum",
    "AttributionModelEnum",
    "AudienceInsightsDimensionEnum",
    "AudienceInsightsMarketingObjectiveEnum",
    "AudienceScopeEnum",
    "AudienceStatusEnum",
    "BatchJobStatusEnum",
    "BidModifierSourceEnum",
    "BiddingSourceEnum",
    "BiddingStrategyStatusEnum",
    "BiddingStrategySystemStatusEnum",
    "BiddingStrategyTypeEnum",
    "BillingSetupStatusEnum",
    "BrandRequestRejectionReasonEnum",
    "BrandSafetySuitabilityEnum",
    "BrandStateEnum",
    "BudgetCampaignAssociationStatusEnum",
    "BudgetDeliveryMethodEnum",
    "BudgetPeriodEnum",
    "BudgetStatusEnum",
    "BudgetTypeEnum",
    "BusinessMessageCallToActionTypeEnum",
    "BusinessMessageProviderEnum",
    "CallConversionReportingStateEnum",
    "CallToActionTypeEnum",
    "CallTrackingDisplayLocationEnum",
    "CallTypeEnum",
    "CampaignCriterionStatusEnum",
    "CampaignDraftStatusEnum",
    "CampaignExperimentTypeEnum",
    "CampaignGroupStatusEnum",
    "CampaignKeywordMatchTypeEnum",
    "CampaignPrimaryStatusEnum",
    "CampaignPrimaryStatusReasonEnum",
    "CampaignServingStatusEnum",
    "CampaignSharedSetStatusEnum",
    "CampaignStatusEnum",
    "ChainRelationshipTypeEnum",
    "ChangeClientTypeEnum",
    "ChangeEventResourceTypeEnum",
    "ChangeStatusOperationEnum",
    "ChangeStatusResourceTypeEnum",
    "ClickTypeEnum",
    "CombinedAudienceStatusEnum",
    "ConsentStatusEnum",
    "ContentLabelTypeEnum",
    "ConversionActionCategoryEnum",
    "ConversionActionCountingTypeEnum",
    "ConversionActionStatusEnum",
    "ConversionActionTypeEnum",
    "ConversionAdjustmentTypeEnum",
    "ConversionAttributionEventTypeEnum",
    "ConversionCustomVariableStatusEnum",
    "ConversionCustomerTypeEnum",
    "ConversionEnvironmentEnum",
    "ConversionLagBucketEnum",
    "ConversionOrAdjustmentLagBucketEnum",
    "ConversionOriginEnum",
    "ConversionTrackingStatusEnum",
    "ConversionValueRulePrimaryDimensionEnum",
    "ConversionValueRuleSetStatusEnum",
    "ConversionValueRuleStatusEnum",
    "ConvertingUserPriorEngagementTypeAndLtvBucketEnum",
    "CriterionCategoryChannelAvailabilityModeEnum",
    "CriterionCategoryLocaleAvailabilityModeEnum",
    "CriterionSystemServingStatusEnum",
    "CriterionTypeEnum",
    "CustomAudienceMemberTypeEnum",
    "CustomAudienceStatusEnum",
    "CustomAudienceTypeEnum",
    "CustomConversionGoalStatusEnum",
    "CustomInterestMemberTypeEnum",
    "CustomInterestStatusEnum",
    "CustomInterestTypeEnum",
    "CustomerAcquisitionOptimizationModeEnum",
    "CustomerMatchUploadKeyTypeEnum",
    "CustomerPayPerConversionEligibilityFailureReasonEnum",
    "CustomerStatusEnum",
    "CustomizerAttributeStatusEnum",
    "CustomizerAttributeTypeEnum",
    "CustomizerValueStatusEnum",
    "DataDrivenModelStatusEnum",
    "DataLinkStatusEnum",
    "DataLinkTypeEnum",
    "DayOfWeekEnum",
    "DemandGenChannelConfigEnum",
    "DemandGenChannelStrategyEnum",
    "DeviceEnum",
    "DisplayAdFormatSettingEnum",
    "DisplayUploadProductTypeEnum",
    "DistanceBucketEnum",
    "EuPoliticalAdvertisingStatusEnum",
    "ExperimentMetricDirectionEnum",
    "ExperimentMetricEnum",
    "ExperimentStatusEnum",
    "ExperimentTypeEnum",
    "ExternalConversionSourceEnum",
    "FixedCpmGoalEnum",
    "FixedCpmTargetFrequencyTimeUnitEnum",
    "FrequencyCapEventTypeEnum",
    "FrequencyCapLevelEnum",
    "FrequencyCapTimeUnitEnum",
    "GenderTypeEnum",
    "GeoTargetConstantStatusEnum",
    "GeoTargetingTypeEnum",
    "GoalConfigLevelEnum",
    "GoogleAdsFieldCategoryEnum",
    "GoogleAdsFieldDataTypeEnum",
    "GoogleVoiceCallStatusEnum",
    "HotelAssetSuggestionStatusEnum",
    "HotelDateSelectionTypeEnum",
    "HotelPriceBucketEnum",
    "HotelRateTypeEnum",
    "HotelReconciliationStatusEnum",
    "IdentityVerificationProgramEnum",
    "IdentityVerificationProgramStatusEnum",
    "IncomeRangeTypeEnum",
    "InsightsKnowledgeGraphEntityCapabilitiesEnum",
    "InsightsTrendEnum",
    "InteractionEventTypeEnum",
    "InteractionTypeEnum",
    "InvoiceTypeEnum",
    "KeywordMatchTypeEnum",
    "KeywordPlanAggregateMetricTypeEnum",
    "KeywordPlanCompetitionLevelEnum",
    "KeywordPlanConceptGroupTypeEnum",
    "KeywordPlanForecastIntervalEnum",
    "KeywordPlanKeywordAnnotationEnum",
    "KeywordPlanNetworkEnum",
    "LabelStatusEnum",
    "LeadFormCallToActionTypeEnum",
    "LeadFormDesiredIntentEnum",
    "LeadFormFieldUserInputTypeEnum",
    "LeadFormPostSubmitCallToActionTypeEnum",
    "LegacyAppInstallAdAppStoreEnum",
    "LinkedAccountTypeEnum",
    "LinkedProductTypeEnum",
    "ListingGroupFilterCustomAttributeIndexEnum",
    "ListingGroupFilterListingSourceEnum",
    "ListingGroupFilterProductCategoryLevelEnum",
    "ListingGroupFilterProductChannelEnum",
    "ListingGroupFilterProductConditionEnum",
    "ListingGroupFilterProductTypeLevelEnum",
    "ListingGroupFilterTypeEnum",
    "ListingGroupTypeEnum",
    "ListingTypeEnum",
    "LocalServicesBusinessRegistrationCheckRejectionReasonEnum",
    "LocalServicesBusinessRegistrationTypeEnum",
    "LocalServicesCreditStateEnum",
    "LocalServicesEmployeeStatusEnum",
    "LocalServicesEmployeeTypeEnum",
    "LocalServicesInsuranceRejectionReasonEnum",
    "LocalServicesLeadConversationTypeEnum",
    "LocalServicesLeadCreditIssuanceDecisionEnum",
    "LocalServicesLeadStatusEnum",
    "LocalServicesLeadSurveyAnswerEnum",
    "LocalServicesLeadSurveyDissatisfiedReasonEnum",
    "LocalServicesLeadSurveySatisfiedReasonEnum",
    "LocalServicesLeadTypeEnum",
    "LocalServicesLicenseRejectionReasonEnum",
    "LocalServicesParticipantTypeEnum",
    "LocalServicesVerificationArtifactStatusEnum",
    "LocalServicesVerificationArtifactTypeEnum",
    "LocalServicesVerificationStatusEnum",
    "LocationGroupRadiusUnitsEnum",
    "LocationOwnershipTypeEnum",
    "LocationSourceTypeEnum",
    "LocationStringFilterTypeEnum",
    "LookalikeExpansionLevelEnum",
    "ManagerLinkStatusEnum",
    "MediaTypeEnum",
    "MimeTypeEnum",
    "MinuteOfHourEnum",
    "MobileAppVendorEnum",
    "MobileDeviceTypeEnum",
    "MonthOfYearEnum",
    "NegativeGeoTargetTypeEnum",
    "NonSkippableMaxDurationEnum",
    "NonSkippableMinDurationEnum",
    "OfflineConversionDiagnosticStatusEnum",
    "OfflineEventUploadClientEnum",
    "OfflineUserDataJobFailureReasonEnum",
    "OfflineUserDataJobMatchRateRangeEnum",
    "OfflineUserDataJobStatusEnum",
    "OfflineUserDataJobTypeEnum",
    "OperatingSystemVersionOperatorTypeEnum",
    "OptimizationGoalTypeEnum",
    "ParentalStatusTypeEnum",
    "PaymentModeEnum",
    "PerformanceMaxUpgradeStatusEnum",
    "PlacementTypeEnum",
    "PolicyApprovalStatusEnum",
    "PolicyReviewStatusEnum",
    "PolicyTopicEntryTypeEnum",
    "PolicyTopicEvidenceDestinationMismatchUrlTypeEnum",
    "PolicyTopicEvidenceDestinationNotWorkingDeviceEnum",
    "PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum",
    "PositiveGeoTargetTypeEnum",
    "PriceExtensionPriceQualifierEnum",
    "PriceExtensionPriceUnitEnum",
    "PriceExtensionTypeEnum",
    "ProductAvailabilityEnum",
    "ProductCategoryLevelEnum",
    "ProductCategoryStateEnum",
    "ProductChannelEnum",
    "ProductChannelExclusivityEnum",
    "ProductConditionEnum",
    "ProductCustomAttributeIndexEnum",
    "ProductIssueSeverityEnum",
    "ProductLinkInvitationStatusEnum",
    "ProductStatusEnum",
    "ProductTypeLevelEnum",
    "PromotionExtensionDiscountModifierEnum",
    "PromotionExtensionOccasionEnum",
    "ProximityRadiusUnitsEnum",
    "QualityScoreBucketEnum",
    "ReachPlanAgeRangeEnum",
    "ReachPlanConversionRateModelEnum",
    "ReachPlanNetworkEnum",
    "ReachPlanPlannableUserListStatusEnum",
    "ReachPlanSurfaceEnum",
    "RecommendationSubscriptionStatusEnum",
    "RecommendationTypeEnum",
    "ResourceChangeOperationEnum",
    "ResourceLimitTypeEnum",
    "ResponseContentTypeEnum",
    "SearchEngineResultsPageTypeEnum",
    "SearchTermMatchTypeEnum",
    "SearchTermTargetingStatusEnum",
    "SeasonalityEventScopeEnum",
    "SeasonalityEventStatusEnum",
    "ServedAssetFieldTypeEnum",
    "SharedSetStatusEnum",
    "SharedSetTypeEnum",
    "ShoppingAddProductsToCampaignRecommendationEnum",
    "SimulationModificationMethodEnum",
    "SimulationTypeEnum",
    "SkAdNetworkAdEventTypeEnum",
    "SkAdNetworkAttributionCreditEnum",
    "SkAdNetworkCoarseConversionValueEnum",
    "SkAdNetworkSourceTypeEnum",
    "SkAdNetworkUserTypeEnum",
    "SlotEnum",
    "SmartCampaignNotEligibleReasonEnum",
    "SmartCampaignStatusEnum",
    "SpendingLimitTypeEnum",
    "SummaryRowSettingEnum",
    "SystemManagedResourceSourceEnum",
    "TargetCpaOptInRecommendationGoalEnum",
    "TargetFrequencyTimeUnitEnum",
    "TargetImpressionShareLocationEnum",
    "TargetingDimensionEnum",
    "TimeTypeEnum",
    "TrackingCodePageFormatEnum",
    "TrackingCodeTypeEnum",
    "UserIdentifierSourceEnum",
    "UserInterestTaxonomyTypeEnum",
    "UserListAccessStatusEnum",
    "UserListClosingReasonEnum",
    "UserListCrmDataSourceTypeEnum",
    "UserListCustomerTypeCategoryEnum",
    "UserListDateRuleItemOperatorEnum",
    "UserListFlexibleRuleOperatorEnum",
    "UserListLogicalRuleOperatorEnum",
    "UserListMembershipStatusEnum",
    "UserListNumberRuleItemOperatorEnum",
    "UserListPrepopulationStatusEnum",
    "UserListRuleTypeEnum",
    "UserListSizeRangeEnum",
    "UserListStringRuleItemOperatorEnum",
    "UserListTypeEnum",
    "ValueRuleDeviceTypeEnum",
    "ValueRuleGeoLocationMatchTypeEnum",
    "ValueRuleOperationEnum",
    "ValueRuleSetAttachmentTypeEnum",
    "ValueRuleSetDimensionEnum",
    "VanityPharmaDisplayUrlModeEnum",
    "VanityPharmaTextEnum",
    "VideoAdFormatRestrictionEnum",
    "VideoThumbnailEnum",
    "WebpageConditionOperandEnum",
    "WebpageConditionOperatorEnum",
)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "DistanceBucketEnum",
    },
)


class DistanceBucketEnum(proto.Message):
    r"""Container for distance buckets of a user's distance from an
    advertiser's location extension.

    """

    class DistanceBucket(proto.Enum):
        r"""The distance bucket for a user's distance from an
        advertiser's location extension.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            WITHIN_700M (2):
                User was within 700m of the location.
            WITHIN_1KM (3):
                User was within 1KM of the location.
            WITHIN_5KM (4):
                User was within 5KM of the location.
            WITHIN_10KM (5):
                User was within 10KM of the location.
            WITHIN_15KM (6):
                User was within 15KM of the location.
            WITHIN_20KM (7):
                User was within 20KM of the location.
            WITHIN_25KM (8):
                User was within 25KM of the location.
            WITHIN_30KM (9):
                User was within 30KM of the location.
            WITHIN_35KM (10):
                User was within 35KM of the location.
            WITHIN_40KM (11):
                User was within 40KM of the location.
            WITHIN_45KM (12):
                User was within 45KM of the location.
            WITHIN_50KM (13):
                User was within 50KM of the location.
            WITHIN_55KM (14):
                User was within 55KM of the location.
            WITHIN_60KM (15):
                User was within 60KM of the location.
            WITHIN_65KM (16):
                User was within 65KM of the location.
            BEYOND_65KM (17):
                User was beyond 65KM of the location.
            WITHIN_0_7MILES (18):
                User was within 0.7 miles of the location.
            WITHIN_1MILE (19):
                User was within 1 mile of the location.
            WITHIN_5MILES (20):
                User was within 5 miles of the location.
            WITHIN_10MILES (21):
                User was within 10 miles of the location.
            WITHIN_15MILES (22):
                User was within 15 miles of the location.
            WITHIN_20MILES (23):
                User was within 20 miles of the location.
            WITHIN_25MILES (24):
                User was within 25 miles of the location.
            WITHIN_30MILES (25):
                User was within 30 miles of the location.
            WITHIN_35MILES (26):
                User was within 35 miles of the location.
            WITHIN_40MILES (27):
                User was within 40 miles of the location.
            BEYOND_40MILES (28):
                User was beyond 40 miles of the location.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        WITHIN_700M = 2
        WITHIN_1KM = 3
        WITHIN_5KM = 4
        WITHIN_10KM = 5
        WITHIN_15KM = 6
        WITHIN_20KM = 7
        WITHIN_25KM = 8
        WITHIN_30KM = 9
        WITHIN_35KM = 10
        WITHIN_40KM = 11
        WITHIN_45KM = 12
        WITHIN_50KM = 13
        WITHIN_55KM = 14
        WITHIN_60KM = 15
        WITHIN_65KM = 16
        BEYOND_65KM = 17
        WITHIN_0_7MILES = 18
        WITHIN_1MILE = 19
        WITHIN_5MILES = 20
        WITHIN_10MILES = 21
        WITHIN_15MILES = 22
        WITHIN_20MILES = 23
        WITHIN_25MILES = 24
        WITHIN_30MILES = 25
        WITHIN_35MILES = 26
        WITHIN_40MILES = 27
        BEYOND_40MILES = 28


__all__ = tuple(sorted(__protobuf__.manifest))



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""A custom gRPC Interceptor that logs requests and responses to Cloud Logging.

The custom interceptor object is passed into the get_service method of the
GoogleAdsClient. It intercepts requests and responses, parses them into a
human readable structure and logs them using the logging service instantiated
within the class (in this case, a Cloud Logging client).
"""

import time
from typing import Any, Callable, Dict, Optional

from google.cloud import logging as google_cloud_logging
from grpc._interceptor import _ClientCallDetails

from google.ads.googleads.interceptors import LoggingInterceptor


class CloudLoggingInterceptor(LoggingInterceptor):
    """An interceptor that logs rpc request and response details to Google Cloud Logging.

    This class inherits logic from the LoggingInterceptor, which simplifies the
    implementation here. Some logic is required here in order to make the
    underlying logic work -- comments make note of this where applicable.
    NOTE: Inheriting from the LoggingInterceptor class could yield unexpected side
    effects. For example, if the LoggingInterceptor class is updated, this class would
    inherit the updated logic, which could affect its functionality. One option to avoid
    this is to inherit from the Interceptor class instead, and selectively copy whatever
    logic is needed from the LoggingInterceptor class."""

    def __init__(self, api_version: str):
        """Initializer for the CloudLoggingInterceptor.

        Args:
            api_version: a str of the API version of the request.
        """
        super().__init__(logger=None, api_version=api_version)
        # Instantiate the Cloud Logging client.
        logging_client: google_cloud_logging.Client = google_cloud_logging.Client()
        self.logger: google_cloud_logging.Logger = logging_client.logger("cloud_logging")
        self.rpc_start: float
        self.rpc_end: float

    def log_successful_request(
        self,
        method: str,
        customer_id: Optional[str],
        metadata_json: str,
        request_id: str,
        request: Any,  # google.ads.googleads.vX.services.types.SearchGoogleAdsRequest or SearchGoogleAdsStreamRequest
        trailing_metadata_json: str,
        response: Any,  # grpc.Call or grpc.Future
    ) -> None:
        """Handles logging of a successful request.

        Args:
            method: The method of the request.
            customer_id: The customer ID associated with the request.
            metadata_json: A JSON str of initial_metadata.
            request_id: A unique ID for the request provided in the response.
            request: An instance of a request proto message.
            trailing_metadata_json: A JSON str of trailing_metadata.
            response: A grpc.Call/grpc.Future instance.
        """
        # Retrieve and mask the RPC result from the response future.
        # This method is available from the LoggingInterceptor class.
        # Ensure self._cache is set in order for this to work.
        # The response result could contain up to 10,000 rows of data,
        # so consider truncating this value before logging it, to save
        # on data storage costs and maintain readability.
        result: Any = self.retrieve_and_mask_result(response)

        # elapsed_ms is the approximate elapsed time of the RPC, in milliseconds.
        # There are different ways to define and measure elapsed time, so use
        # whatever approach makes sense for your monitoring purposes.
        # rpc_start and rpc_end are set in the intercept_unary_* methods below.
        elapsed_ms: float = (self.rpc_end - self.rpc_start) * 1000

        debug_log: Dict[str, Any] = {
            "method": method,
            "host": metadata_json,
            "request_id": request_id,
            "request": str(request),
            "headers": trailing_metadata_json,
            "response": str(result),
            "is_fault": False,
            "elapsed_ms": elapsed_ms,
        }
        self.logger.log_struct(debug_log, severity="DEBUG")

        info_log: Dict[str, Any] = {
            "customer_id": customer_id,
            "method": method,
            "request_id": request_id,
            "is_fault": False,
            # Available from the Interceptor class.
            "api_version": self._api_version,
        }
        self.logger.log_struct(info_log, severity="INFO")

    def log_failed_request(
        self,
        method: str,
        customer_id: Optional[str],
        metadata_json: str,
        request_id: str,
        request: Any,  # google.ads.googleads.vX.services.types.SearchGoogleAdsRequest or SearchGoogleAdsStreamRequest
        trailing_metadata_json: str,
        response: Any,  # grpc.Call or grpc.Future
    ) -> None:
        """Handles logging of a failed request.

        Args:
            method: The method of the request.
            customer_id: The customer ID associated with the request.
            metadata_json: A JSON str of initial_metadata.
            request_id: A unique ID for the request provided in the response.
            request: An instance of a request proto message.
            trailing_metadata_json: A JSON str of trailing_metadata.
            response: A JSON str of the response message.
        """
        exception: Any = self._get_error_from_response(response)
        exception_str: str = self._parse_exception_to_str(exception)
        fault_message: str = self._get_fault_message(exception)

        info_log: Dict[str, Any] = {
            "method": method,
            "endpoint": self.endpoint,
            "host": metadata_json,
            "request_id": request_id,
            "request": str(request),
            "headers": trailing_metadata_json,
            "exception": exception_str,
            "is_fault": True,
        }
        self.logger.log_struct(info_log, severity="INFO")

        error_log: Dict[str, Any] = {
            "method": method,
            "endpoint": self.endpoint,
            "request_id": request_id,
            "customer_id": customer_id,
            "is_fault": True,
            "fault_message": fault_message,
        }
        self.logger.log_struct(error_log, severity="ERROR")

    def intercept_unary_unary(
        self,
        continuation: Callable[[_ClientCallDetails, Any], Any], # Any is request type
        client_call_details: _ClientCallDetails,
        request: Any,  # google.ads.googleads.vX.services.types.SearchGoogleAdsRequest
    ) -> Any:  # grpc.Call or grpc.Future
        """Intercepts and logs API interactions.

        Overrides abstract method defined in grpc.UnaryUnaryClientInterceptor.

        Args:
            continuation: a function to continue the request process.
            client_call_details: a grpc._interceptor._ClientCallDetails
                instance containing request metadata.
            request: a SearchGoogleAdsRequest or SearchGoogleAdsStreamRequest
                message class instance.

        Returns:
            A grpc.Call/grpc.Future instance representing a service response.
        """
        # Set the rpc_end value to current time when RPC completes.
        def update_rpc_end(response_future: Any) -> None: # response_future is grpc.Future
            self.rpc_end = time.perf_counter()

        # Capture precise clock time to later calculate approximate elapsed
        # time of the RPC.
        self.rpc_start = time.perf_counter()

        # The below call is REQUIRED.
        response: Any = continuation(client_call_details, request) # response is grpc.Call or grpc.Future

        response.add_done_callback(update_rpc_end)

        self.log_request(client_call_details, request, response)

        # The below return is REQUIRED.
        return response

    def intercept_unary_stream(
        self,
        continuation: Callable[[_ClientCallDetails, Any], Any], # Any is request type
        client_call_details: _ClientCallDetails,
        request: Any,  # google.ads.googleads.vX.services.types.SearchGoogleAdsStreamRequest
    ) -> Any:  # grpc.Call or grpc.Future
        """Intercepts and logs API interactions for Unary-Stream requests.

        Overrides abstract method defined in grpc.UnaryStreamClientInterceptor.

        Args:
            continuation: a function to continue the request process.
            client_call_details: a grpc._interceptor._ClientCallDetails
                instance containing request metadata.
            request: a SearchGoogleAdsRequest or SearchGoogleAdsStreamRequest
                message class instance.

        Returns:
            A grpc.Call/grpc.Future instance representing a service response.
        """

        def on_rpc_complete(response_future: Any) -> None: # response_future is grpc.Future
            self.rpc_end = time.perf_counter()
            self.log_request(client_call_details, request, response_future)

        # Capture precise clock time to later calculate approximate elapsed
        # time of the RPC.
        self.rpc_start = time.perf_counter()

        # The below call is REQUIRED.
        response: Any = continuation(client_call_details, request) # response is grpc.Call or grpc.Future

        # Set self._cache to the cache on the response wrapper in order to
        # access the streaming logs. This is REQUIRED in order to log streaming
        # requests.
        self._cache = response.get_cache()

        response.add_done_callback(on_rpc_complete)

        # The below return is REQUIRED.
        return response



#!/usr/bin/env python
# Copyright 2018 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example illustrates how to retrieve ad group bid modifiers."""


import argparse
import sys
from typing import Optional

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.resources.types.ad_group_bid_modifier import (
    AdGroupBidModifier,
)
from google.ads.googleads.v22.services.services.google_ads_service import (
    GoogleAdsServiceClient,
)
from google.ads.googleads.v22.services.types.google_ads_service import (
    GoogleAdsRow,
    SearchGoogleAdsRequest,
    SearchGoogleAdsResponse,
)


def main(
    client: GoogleAdsClient, customer_id: str, ad_group_id: Optional[str] = None
) -> None:
    ga_service: GoogleAdsServiceClient = client.get_service("GoogleAdsService")

    query: str = """
        SELECT
          campaign.id,
          ad_group.id,
          ad_group_bid_modifier.criterion_id,
          ad_group_bid_modifier.bid_modifier,
          ad_group_bid_modifier.device.type,
          ad_group_bid_modifier.hotel_date_selection_type.type,
          ad_group_bid_modifier.hotel_advance_booking_window.min_days,
          ad_group_bid_modifier.hotel_advance_booking_window.max_days,
          ad_group_bid_modifier.hotel_length_of_stay.min_nights,
          ad_group_bid_modifier.hotel_length_of_stay.max_nights,
          ad_group_bid_modifier.hotel_check_in_day.day_of_week,
          ad_group_bid_modifier.hotel_check_in_date_range.start_date,
          ad_group_bid_modifier.hotel_check_in_date_range.end_date
        FROM ad_group_bid_modifier"""

    if ad_group_id:
        query += f" WHERE ad_group.id = {ad_group_id}"

    # Limit results to 10,000 rows as the number of bid modifiers can be large.
    query += " LIMIT 10000"

    search_request: SearchGoogleAdsRequest = client.get_type(
        "SearchGoogleAdsRequest"
    )
    search_request.customer_id = customer_id
    search_request.query = query

    results: SearchGoogleAdsResponse = ga_service.search(request=search_request)

    row: GoogleAdsRow
    for row in results:
        modifier: AdGroupBidModifier = row.ad_group_bid_modifier
        print(
            "Ad group bid modifier with criterion ID "
            f"'{modifier.criterion_id}', bid modifier value "
            f"'{modifier.bid_modifier or 0.00}', device type "
            f"'{modifier.device.type_.name}' was found in ad group with ID "
            f"'{row.ad_group.id}' of campaign with ID '{row.campaign.id}'."
        )

        criterion_field: str = (
            type(modifier).pb(modifier).WhichOneof("criterion")
        )
        criterion_details: str = f"  - Criterion type: {criterion_field}, "

        if criterion_field == "device":
            criterion_details += f"Type: {modifier.device.type_}"
        elif criterion_field == "hotel_advance_booking_window":
            criterion_details += (
                f"Min Days: {modifier.hotel_advance_booking_window.min_days}, "
                f"Max Days: {modifier.hotel_advance_booking_window.max_days}"
            )
        elif criterion_field == "hotel_check_in_day":
            criterion_details += (
                f"Day of the week: {modifier.hotel_check_in_day.day_of_week}"
            )
        elif criterion_field == "hotel_date_selection_type":
            criterion_details += f"Date selection type: {modifier.hotel_date_selection_type.type_}"
        elif criterion_field == "hotel_length_of_stay":
            criterion_details += (
                f"Min Nights: {modifier.hotel_length_of_stay.min_nights}, "
                f"Max Nights: {modifier.hotel_length_of_stay.max_nights}"
            )
        else:
            criterion_details = "  - No Criterion type found."

        print(criterion_details)


if __name__ == "__main__":
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description="List ad group bid modifiers for specified customer."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-a",
        "--ad_group_id",
        type=str,
        required=False,
        help=(
            "The ad group ID. Specify this to list ad group "
            "bid modifiers solely for this ad group ID."
        ),
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(
            googleads_client,
            args.customer_id,
            ad_group_id=args.ad_group_id,
        )
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.resources",
    marshal="google.ads.googleads.v21",
    manifest={
        "AdParameter",
    },
)


class AdParameter(proto.Message):
    r"""An ad parameter that is used to update numeric values (such as
    prices or inventory levels) in any text line of an ad (including
    URLs). There can be a maximum of two AdParameters per ad group
    criterion. (One with parameter_index = 1 and one with
    parameter_index = 2.) In the ad the parameters are referenced by a
    placeholder of the form "{param#:value}". For example,
    "{param1:$17}"


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the ad parameter. Ad
            parameter resource names have the form:

            ``customers/{customer_id}/adParameters/{ad_group_id}~{criterion_id}~{parameter_index}``
        ad_group_criterion (str):
            Immutable. The ad group criterion that this
            ad parameter belongs to.

            This field is a member of `oneof`_ ``_ad_group_criterion``.
        parameter_index (int):
            Immutable. The unique index of this ad
            parameter. Must be either 1 or 2.

            This field is a member of `oneof`_ ``_parameter_index``.
        insertion_text (str):
            Numeric value to insert into the ad text. The
            following restrictions  apply:

             - Can use comma or period as a separator, with
              an optional period or    comma (respectively)
              for fractional values. For example,
              1,000,000.00    and 2.000.000,10 are valid.
             - Can be prepended or appended with a currency
              symbol. For example,    $99.99 is valid.
             - Can be prepended or appended with a currency
              code. For example, 99.99USD    and EUR200 are
              valid.
             - Can use '%'. For example, 1.0% and 1,0% are
              valid.
             - Can use plus or minus. For example, -10.99
              and 25+ are valid.
             - Can use '/' between two numbers. For example
              4/1 and 0.95/0.45 are    valid.

            This field is a member of `oneof`_ ``_insertion_text``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    ad_group_criterion: str = proto.Field(
        proto.STRING,
        number=5,
        optional=True,
    )
    parameter_index: int = proto.Field(
        proto.INT64,
        number=6,
        optional=True,
    )
    insertion_text: str = proto.Field(
        proto.STRING,
        number=7,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.errors",
    marshal="google.ads.googleads.v19",
    manifest={
        "AccountBudgetProposalErrorEnum",
    },
)


class AccountBudgetProposalErrorEnum(proto.Message):
    r"""Container for enum describing possible account budget
    proposal errors.

    """

    class AccountBudgetProposalError(proto.Enum):
        r"""Enum describing possible account budget proposal errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            FIELD_MASK_NOT_ALLOWED (2):
                The field mask must be empty for
                create/end/remove proposals.
            IMMUTABLE_FIELD (3):
                The field cannot be set because of the
                proposal type.
            REQUIRED_FIELD_MISSING (4):
                The field is required because of the proposal
                type.
            CANNOT_CANCEL_APPROVED_PROPOSAL (5):
                Proposals that have been approved cannot be
                cancelled.
            CANNOT_REMOVE_UNAPPROVED_BUDGET (6):
                Budgets that haven't been approved cannot be
                removed.
            CANNOT_REMOVE_RUNNING_BUDGET (7):
                Budgets that are currently running cannot be
                removed.
            CANNOT_END_UNAPPROVED_BUDGET (8):
                Budgets that haven't been approved cannot be
                truncated.
            CANNOT_END_INACTIVE_BUDGET (9):
                Only budgets that are currently running can
                be truncated.
            BUDGET_NAME_REQUIRED (10):
                All budgets must have names.
            CANNOT_UPDATE_OLD_BUDGET (11):
                Expired budgets cannot be edited after a
                sufficient amount of time has passed.
            CANNOT_END_IN_PAST (12):
                It is not permissible a propose a new budget
                that ends in the past.
            CANNOT_EXTEND_END_TIME (13):
                An expired budget cannot be extended to
                overlap with the running budget.
            PURCHASE_ORDER_NUMBER_REQUIRED (14):
                A purchase order number is required.
            PENDING_UPDATE_PROPOSAL_EXISTS (15):
                Budgets that have a pending update cannot be
                updated.
            MULTIPLE_BUDGETS_NOT_ALLOWED_FOR_UNAPPROVED_BILLING_SETUP (16):
                Cannot propose more than one budget when the
                corresponding billing setup hasn't been
                approved.
            CANNOT_UPDATE_START_TIME_FOR_STARTED_BUDGET (17):
                Cannot update the start time of a budget that
                has already started.
            SPENDING_LIMIT_LOWER_THAN_ACCRUED_COST_NOT_ALLOWED (18):
                Cannot update the spending limit of a budget
                with an amount lower than what has already been
                spent.
            UPDATE_IS_NO_OP (19):
                Cannot propose a budget update without
                actually changing any fields.
            END_TIME_MUST_FOLLOW_START_TIME (20):
                The end time must come after the start time.
            BUDGET_DATE_RANGE_INCOMPATIBLE_WITH_BILLING_SETUP (21):
                The budget's date range must fall within the
                date range of its billing setup.
            NOT_AUTHORIZED (22):
                The user is not authorized to mutate budgets
                for the given billing setup.
            INVALID_BILLING_SETUP (23):
                Mutates are not allowed for the given billing
                setup.
            OVERLAPS_EXISTING_BUDGET (24):
                Budget creation failed as it overlaps with a
                pending budget proposal or an approved budget.
            CANNOT_CREATE_BUDGET_THROUGH_API (25):
                The control setting in user's payments
                profile doesn't allow budget creation through
                API. Log in to Google Ads to create budget.
            INVALID_MASTER_SERVICE_AGREEMENT (26):
                Master service agreement has not been signed
                yet for the Payments Profile.
            CANCELED_BILLING_SETUP (27):
                Budget mutates are not allowed because the
                given billing setup is canceled.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        FIELD_MASK_NOT_ALLOWED = 2
        IMMUTABLE_FIELD = 3
        REQUIRED_FIELD_MISSING = 4
        CANNOT_CANCEL_APPROVED_PROPOSAL = 5
        CANNOT_REMOVE_UNAPPROVED_BUDGET = 6
        CANNOT_REMOVE_RUNNING_BUDGET = 7
        CANNOT_END_UNAPPROVED_BUDGET = 8
        CANNOT_END_INACTIVE_BUDGET = 9
        BUDGET_NAME_REQUIRED = 10
        CANNOT_UPDATE_OLD_BUDGET = 11
        CANNOT_END_IN_PAST = 12
        CANNOT_EXTEND_END_TIME = 13
        PURCHASE_ORDER_NUMBER_REQUIRED = 14
        PENDING_UPDATE_PROPOSAL_EXISTS = 15
        MULTIPLE_BUDGETS_NOT_ALLOWED_FOR_UNAPPROVED_BILLING_SETUP = 16
        CANNOT_UPDATE_START_TIME_FOR_STARTED_BUDGET = 17
        SPENDING_LIMIT_LOWER_THAN_ACCRUED_COST_NOT_ALLOWED = 18
        UPDATE_IS_NO_OP = 19
        END_TIME_MUST_FOLLOW_START_TIME = 20
        BUDGET_DATE_RANGE_INCOMPATIBLE_WITH_BILLING_SETUP = 21
        NOT_AUTHORIZED = 22
        INVALID_BILLING_SETUP = 23
        OVERLAPS_EXISTING_BUDGET = 24
        CANNOT_CREATE_BUDGET_THROUGH_API = 25
        INVALID_MASTER_SERVICE_AGREEMENT = 26
        CANCELED_BILLING_SETUP = 27


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.errors",
    marshal="google.ads.googleads.v22",
    manifest={
        "AssetSetErrorEnum",
    },
)


class AssetSetErrorEnum(proto.Message):
    r"""Container for enum describing possible asset set errors."""

    class AssetSetError(proto.Enum):
        r"""Enum describing possible asset set errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            DUPLICATE_ASSET_SET_NAME (2):
                The asset set name matches that of another
                enabled asset set.
            INVALID_PARENT_ASSET_SET_TYPE (3):
                The type of AssetSet.asset_set_source does not match the
                type of AssetSet.location_set.source in its parent AssetSet.
            ASSET_SET_SOURCE_INCOMPATIBLE_WITH_PARENT_ASSET_SET (4):
                The asset set source doesn't match its parent
                AssetSet's data.
            ASSET_SET_TYPE_CANNOT_BE_LINKED_TO_CUSTOMER (5):
                This AssetSet type cannot be linked to
                CustomerAssetSet.
            INVALID_CHAIN_IDS (6):
                The chain id(s) in ChainSet of a LOCATION_SYNC typed
                AssetSet is invalid.
            LOCATION_SYNC_ASSET_SET_DOES_NOT_SUPPORT_RELATIONSHIP_TYPE (7):
                The relationship type in ChainSet of a LOCATION_SYNC typed
                AssetSet is not supported.
            NOT_UNIQUE_ENABLED_LOCATION_SYNC_TYPED_ASSET_SET (8):
                There is more than one enabled LocationSync
                typed AssetSet under one customer.
            INVALID_PLACE_IDS (9):
                The place id(s) in a LocationSync typed
                AssetSet is invalid and can't be decoded.
            OAUTH_INFO_INVALID (11):
                The Google Business Profile OAuth info is
                invalid.
            OAUTH_INFO_MISSING (12):
                The Google Business Profile OAuth info is
                missing.
            CANNOT_DELETE_AS_ENABLED_LINKAGES_EXIST (10):
                Can't delete an AssetSet if it has any
                enabled linkages (e.g. CustomerAssetSet), or
                AssetSet is a parent AssetSet and has enabled
                child AssetSet associated.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        DUPLICATE_ASSET_SET_NAME = 2
        INVALID_PARENT_ASSET_SET_TYPE = 3
        ASSET_SET_SOURCE_INCOMPATIBLE_WITH_PARENT_ASSET_SET = 4
        ASSET_SET_TYPE_CANNOT_BE_LINKED_TO_CUSTOMER = 5
        INVALID_CHAIN_IDS = 6
        LOCATION_SYNC_ASSET_SET_DOES_NOT_SUPPORT_RELATIONSHIP_TYPE = 7
        NOT_UNIQUE_ENABLED_LOCATION_SYNC_TYPED_ASSET_SET = 8
        INVALID_PLACE_IDS = 9
        OAUTH_INFO_INVALID = 11
        OAUTH_INFO_MISSING = 12
        CANNOT_DELETE_AS_ENABLED_LINKAGES_EXIST = 10


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v20.common.types import tag_snippet


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.resources",
    marshal="google.ads.googleads.v20",
    manifest={
        "RemarketingAction",
    },
)


class RemarketingAction(proto.Message):
    r"""A remarketing action. A snippet of JavaScript code that will
    collect the product id and the type of page people visited
    (product page, shopping cart page, purchase page, general site
    visit) on an advertiser's website.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the remarketing action.
            Remarketing action resource names have the form:

            ``customers/{customer_id}/remarketingActions/{remarketing_action_id}``
        id (int):
            Output only. Id of the remarketing action.

            This field is a member of `oneof`_ ``_id``.
        name (str):
            The name of the remarketing action.

            This field is required and should not be empty
            when creating new remarketing actions.

            This field is a member of `oneof`_ ``_name``.
        tag_snippets (MutableSequence[google.ads.googleads.v20.common.types.TagSnippet]):
            Output only. The snippets used for tracking
            remarketing actions.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=5,
        optional=True,
    )
    name: str = proto.Field(
        proto.STRING,
        number=6,
        optional=True,
    )
    tag_snippets: MutableSequence[tag_snippet.TagSnippet] = proto.RepeatedField(
        proto.MESSAGE,
        number=4,
        message=tag_snippet.TagSnippet,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "ExperimentStatusEnum",
    },
)


class ExperimentStatusEnum(proto.Message):
    r"""Container for enum describing the experiment status."""

    class ExperimentStatus(proto.Enum):
        r"""The status of the experiment.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                The value is unknown in this version.
            ENABLED (2):
                The experiment is enabled.
            REMOVED (3):
                The experiment has been removed.
            HALTED (4):
                The experiment has been halted.
                This status can be set from ENABLED status
                through API.
            PROMOTED (5):
                The experiment will be promoted out of
                experimental status.
            SETUP (6):
                Initial status of the experiment.
            INITIATED (7):
                The experiment's campaigns are pending
                materialization. This status can be set from
                SETUP status through API.
            GRADUATED (8):
                The experiment has been graduated.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        ENABLED = 2
        REMOVED = 3
        HALTED = 4
        PROMOTED = 5
        SETUP = 6
        INITIATED = 7
        GRADUATED = 8


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "OfflineConversionDiagnosticStatusEnum",
    },
)


class OfflineConversionDiagnosticStatusEnum(proto.Message):
    r"""All possible statuses for oci diagnostics."""

    class OfflineConversionDiagnosticStatus(proto.Enum):
        r"""Possible statuses of the offline ingestion setup.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            EXCELLENT (2):
                Your offline data ingestion setup is active
                and optimal for downstream processing.
            GOOD (3):
                Your offline ingestion setup is active, but
                there are further improvements you could make.
                See alerts.
            NEEDS_ATTENTION (4):
                Your offline ingestion setup is active, but
                there are errors that require your attention.
                See alerts.
            NO_RECENT_UPLOAD (6):
                Your offline ingestion setup has not received
                data in the last 28 days, there may be something
                wrong.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        EXCELLENT = 2
        GOOD = 3
        NEEDS_ATTENTION = 4
        NO_RECENT_UPLOAD = 6


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "EuPoliticalAdvertisingStatusEnum",
    },
)


class EuPoliticalAdvertisingStatusEnum(proto.Message):
    r"""Container for enum describing whether or not the campaign
    contains political advertising targeted towards the European
    Union.

    """

    class EuPoliticalAdvertisingStatus(proto.Enum):
        r"""Possible values describing whether or not the campaign
        contains political advertising targeted towards the European
        Union.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            CONTAINS_EU_POLITICAL_ADVERTISING (2):
                The campaign contains political advertising
                targeted towards the EU. The campaign will be
                restricted from serving ads in the EU.
            DOES_NOT_CONTAIN_EU_POLITICAL_ADVERTISING (3):
                The campaign does not contain political
                advertising targeted towards the EU. No
                additional serving restrictions will apply.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CONTAINS_EU_POLITICAL_ADVERTISING = 2
        DOES_NOT_CONTAIN_EU_POLITICAL_ADVERTISING = 3


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v20.resources.types import customer_lifecycle_goal
from google.protobuf import field_mask_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.services",
    marshal="google.ads.googleads.v20",
    manifest={
        "ConfigureCustomerLifecycleGoalsRequest",
        "CustomerLifecycleGoalOperation",
        "ConfigureCustomerLifecycleGoalsResponse",
        "ConfigureCustomerLifecycleGoalsResult",
    },
)


class ConfigureCustomerLifecycleGoalsRequest(proto.Message):
    r"""Request message for
    [CustomerLifecycleGoalService.configureCustomerLifecycleGoals][].

    Attributes:
        customer_id (str):
            Required. The ID of the customer performing
            the upload.
        operation (google.ads.googleads.v20.services.types.CustomerLifecycleGoalOperation):
            Required. The operation to perform customer
            lifecycle goal update.
        validate_only (bool):
            Optional. If true, the request is validated
            but not executed. Only errors are returned, not
            results.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operation: "CustomerLifecycleGoalOperation" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="CustomerLifecycleGoalOperation",
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=3,
    )


class CustomerLifecycleGoalOperation(proto.Message):
    r"""A single operation on a customer lifecycle goal.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Optional. FieldMask that determines which
            resource fields are modified in an update.
        create (google.ads.googleads.v20.resources.types.CustomerLifecycleGoal):
            Create operation: Create a new customer
            lifecycle goal.

            This field is a member of `oneof`_ ``operation``.
        update (google.ads.googleads.v20.resources.types.CustomerLifecycleGoal):
            Update operation: Update an existing customer
            lifecycle goal.

            This field is a member of `oneof`_ ``operation``.
    """

    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )
    create: customer_lifecycle_goal.CustomerLifecycleGoal = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="operation",
        message=customer_lifecycle_goal.CustomerLifecycleGoal,
    )
    update: customer_lifecycle_goal.CustomerLifecycleGoal = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="operation",
        message=customer_lifecycle_goal.CustomerLifecycleGoal,
    )


class ConfigureCustomerLifecycleGoalsResponse(proto.Message):
    r"""Response message for
    [CustomerLifecycleGoalService.configureCustomerLifecycleGoals][].

    Attributes:
        result (google.ads.googleads.v20.services.types.ConfigureCustomerLifecycleGoalsResult):
            result for the customer lifecycle goal
            configuration.
    """

    result: "ConfigureCustomerLifecycleGoalsResult" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="ConfigureCustomerLifecycleGoalsResult",
    )


class ConfigureCustomerLifecycleGoalsResult(proto.Message):
    r"""The result for the customer lifecycle goal configuration.

    Attributes:
        resource_name (str):
            Returned for the successful operation.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.enums",
    marshal="google.ads.googleads.v19",
    manifest={
        "AdGroupPrimaryStatusReasonEnum",
    },
)


class AdGroupPrimaryStatusReasonEnum(proto.Message):
    r"""Ad Group Primary Status Reason. Provides insight into why an
    ad group is not serving or not serving optimally. These reasons
    are aggregated to determine an overall AdGroupPrimaryStatus.

    """

    class AdGroupPrimaryStatusReason(proto.Enum):
        r"""Possible ad group status reasons.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            CAMPAIGN_REMOVED (2):
                The user-specified campaign status is removed. Contributes
                to AdGroupPrimaryStatus.NOT_ELIGIBLE.
            CAMPAIGN_PAUSED (3):
                The user-specified campaign status is paused. Contributes to
                AdGroupPrimaryStatus.NOT_ELIGIBLE.
            CAMPAIGN_PENDING (4):
                The user-specified time for this campaign to start is in the
                future. Contributes to AdGroupPrimaryStatus.NOT_ELIGIBLE
            CAMPAIGN_ENDED (5):
                The user-specified time for this campaign to end has passed.
                Contributes to AdGroupPrimaryStatus.NOT_ELIGIBLE.
            AD_GROUP_PAUSED (6):
                The user-specified ad group status is paused.
                Contributes to AdGroupPrimaryStatus.PAUSED.
            AD_GROUP_REMOVED (7):
                The user-specified ad group status is
                removed. Contributes to
                AdGroupPrimaryStatus.REMOVED.
            AD_GROUP_INCOMPLETE (8):
                The construction of this ad group is not yet complete.
                Contributes to AdGroupPrimaryStatus.NOT_ELIGIBLE.
            KEYWORDS_PAUSED (9):
                The user-specified keyword statuses in this ad group are all
                paused. Contributes to AdGroupPrimaryStatus.NOT_ELIGIBLE.
            NO_KEYWORDS (10):
                No eligible keywords exist in this ad group. Contributes to
                AdGroupPrimaryStatus.NOT_ELIGIBLE.
            AD_GROUP_ADS_PAUSED (11):
                The user-specified ad group ads statuses in this ad group
                are all paused. Contributes to
                AdGroupPrimaryStatus.NOT_ELIGIBLE.
            NO_AD_GROUP_ADS (12):
                No eligible ad group ads exist in this ad group. Contributes
                to AdGroupPrimaryStatus.NOT_ELIGIBLE.
            HAS_ADS_DISAPPROVED (13):
                Policy status reason when at least one ad is
                disapproved. Contributes to multiple
                AdGroupPrimaryStatus.
            HAS_ADS_LIMITED_BY_POLICY (14):
                Policy status reason when at least one ad is
                limited by policy. Contributes to multiple
                AdGroupPrimaryStatus.
            MOST_ADS_UNDER_REVIEW (15):
                Policy status reason when most ads are
                pending review. Contributes to
                AdGroupPrimaryStatus.PENDING.
            CAMPAIGN_DRAFT (16):
                The AdGroup belongs to a Draft campaign. Contributes to
                AdGroupPrimaryStatus.NOT_ELIGIBLE.
            AD_GROUP_PAUSED_DUE_TO_LOW_ACTIVITY (19):
                Ad group has been paused due to prolonged low
                activity in serving. Contributes to
                AdGroupPrimaryStatus.PAUSED.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CAMPAIGN_REMOVED = 2
        CAMPAIGN_PAUSED = 3
        CAMPAIGN_PENDING = 4
        CAMPAIGN_ENDED = 5
        AD_GROUP_PAUSED = 6
        AD_GROUP_REMOVED = 7
        AD_GROUP_INCOMPLETE = 8
        KEYWORDS_PAUSED = 9
        NO_KEYWORDS = 10
        AD_GROUP_ADS_PAUSED = 11
        NO_AD_GROUP_ADS = 12
        HAS_ADS_DISAPPROVED = 13
        HAS_ADS_LIMITED_BY_POLICY = 14
        MOST_ADS_UNDER_REVIEW = 15
        CAMPAIGN_DRAFT = 16
        AD_GROUP_PAUSED_DUE_TO_LOW_ACTIVITY = 19


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "ConversionLagBucketEnum",
    },
)


class ConversionLagBucketEnum(proto.Message):
    r"""Container for enum representing the number of days between
    impression and conversion.

    """

    class ConversionLagBucket(proto.Enum):
        r"""Enum representing the number of days between impression and
        conversion.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            LESS_THAN_ONE_DAY (2):
                Conversion lag bucket from 0 to 1 day. 0 day
                is included, 1 day is not.
            ONE_TO_TWO_DAYS (3):
                Conversion lag bucket from 1 to 2 days. 1 day
                is included, 2 days is not.
            TWO_TO_THREE_DAYS (4):
                Conversion lag bucket from 2 to 3 days. 2
                days is included, 3 days is not.
            THREE_TO_FOUR_DAYS (5):
                Conversion lag bucket from 3 to 4 days. 3
                days is included, 4 days is not.
            FOUR_TO_FIVE_DAYS (6):
                Conversion lag bucket from 4 to 5 days. 4
                days is included, 5 days is not.
            FIVE_TO_SIX_DAYS (7):
                Conversion lag bucket from 5 to 6 days. 5
                days is included, 6 days is not.
            SIX_TO_SEVEN_DAYS (8):
                Conversion lag bucket from 6 to 7 days. 6
                days is included, 7 days is not.
            SEVEN_TO_EIGHT_DAYS (9):
                Conversion lag bucket from 7 to 8 days. 7
                days is included, 8 days is not.
            EIGHT_TO_NINE_DAYS (10):
                Conversion lag bucket from 8 to 9 days. 8
                days is included, 9 days is not.
            NINE_TO_TEN_DAYS (11):
                Conversion lag bucket from 9 to 10 days. 9
                days is included, 10 days is not.
            TEN_TO_ELEVEN_DAYS (12):
                Conversion lag bucket from 10 to 11 days. 10
                days is included, 11 days is not.
            ELEVEN_TO_TWELVE_DAYS (13):
                Conversion lag bucket from 11 to 12 days. 11
                days is included, 12 days is not.
            TWELVE_TO_THIRTEEN_DAYS (14):
                Conversion lag bucket from 12 to 13 days. 12
                days is included, 13 days is not.
            THIRTEEN_TO_FOURTEEN_DAYS (15):
                Conversion lag bucket from 13 to 14 days. 13
                days is included, 14 days is not.
            FOURTEEN_TO_TWENTY_ONE_DAYS (16):
                Conversion lag bucket from 14 to 21 days. 14
                days is included, 21 days is not.
            TWENTY_ONE_TO_THIRTY_DAYS (17):
                Conversion lag bucket from 21 to 30 days. 21
                days is included, 30 days is not.
            THIRTY_TO_FORTY_FIVE_DAYS (18):
                Conversion lag bucket from 30 to 45 days. 30
                days is included, 45 days is not.
            FORTY_FIVE_TO_SIXTY_DAYS (19):
                Conversion lag bucket from 45 to 60 days. 45
                days is included, 60 days is not.
            SIXTY_TO_NINETY_DAYS (20):
                Conversion lag bucket from 60 to 90 days. 60
                days is included, 90 days is not.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        LESS_THAN_ONE_DAY = 2
        ONE_TO_TWO_DAYS = 3
        TWO_TO_THREE_DAYS = 4
        THREE_TO_FOUR_DAYS = 5
        FOUR_TO_FIVE_DAYS = 6
        FIVE_TO_SIX_DAYS = 7
        SIX_TO_SEVEN_DAYS = 8
        SEVEN_TO_EIGHT_DAYS = 9
        EIGHT_TO_NINE_DAYS = 10
        NINE_TO_TEN_DAYS = 11
        TEN_TO_ELEVEN_DAYS = 12
        ELEVEN_TO_TWELVE_DAYS = 13
        TWELVE_TO_THIRTEEN_DAYS = 14
        THIRTEEN_TO_FOURTEEN_DAYS = 15
        FOURTEEN_TO_TWENTY_ONE_DAYS = 16
        TWENTY_ONE_TO_THIRTY_DAYS = 17
        THIRTY_TO_FORTY_FIVE_DAYS = 18
        FORTY_FIVE_TO_SIXTY_DAYS = 19
        SIXTY_TO_NINETY_DAYS = 20


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "ResponseContentTypeEnum",
    },
)


class ResponseContentTypeEnum(proto.Message):
    r"""Container for possible response content types."""

    class ResponseContentType(proto.Enum):
        r"""Possible response content types.

        Values:
            UNSPECIFIED (0):
                Not specified. Will return the resource name
                only in the response.
            RESOURCE_NAME_ONLY (1):
                The mutate response will be the resource
                name.
            MUTABLE_RESOURCE (2):
                The mutate response will contain the resource
                name and the resource with mutable fields if
                possible. Otherwise, only the resource name will
                be returned.
        """

        UNSPECIFIED = 0
        RESOURCE_NAME_ONLY = 1
        MUTABLE_RESOURCE = 2


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "ExperimentMetricEnum",
    },
)


class ExperimentMetricEnum(proto.Message):
    r"""Container for enum describing the type of experiment metric."""

    class ExperimentMetric(proto.Enum):
        r"""The type of experiment metric.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                The value is unknown in this version.
            CLICKS (2):
                The goal of the experiment is clicks.
            IMPRESSIONS (3):
                The goal of the experiment is impressions.
            COST (4):
                The goal of the experiment is cost.
            CONVERSIONS_PER_INTERACTION_RATE (5):
                The goal of the experiment is conversion
                rate.
            COST_PER_CONVERSION (6):
                The goal of the experiment is cost per
                conversion.
            CONVERSIONS_VALUE_PER_COST (7):
                The goal of the experiment is conversion
                value per cost.
            AVERAGE_CPC (8):
                The goal of the experiment is avg cpc.
            CTR (9):
                The goal of the experiment is ctr.
            INCREMENTAL_CONVERSIONS (10):
                The goal of the experiment is incremental
                conversions.
            COMPLETED_VIDEO_VIEWS (11):
                The goal of the experiment is completed video
                views.
            CUSTOM_ALGORITHMS (12):
                The goal of the experiment is custom
                algorithms.
            CONVERSIONS (13):
                The goal of the experiment is conversions.
            CONVERSION_VALUE (14):
                The goal of the experiment is conversion
                value.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CLICKS = 2
        IMPRESSIONS = 3
        COST = 4
        CONVERSIONS_PER_INTERACTION_RATE = 5
        COST_PER_CONVERSION = 6
        CONVERSIONS_VALUE_PER_COST = 7
        AVERAGE_CPC = 8
        CTR = 9
        INCREMENTAL_CONVERSIONS = 10
        COMPLETED_VIDEO_VIEWS = 11
        CUSTOM_ALGORITHMS = 12
        CONVERSIONS = 13
        CONVERSION_VALUE = 14


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import logging as std_logging
from typing import Callable, MutableSequence, Optional, Sequence, Tuple, Union

from google.ads.googleads.v19 import gapic_version as package_version

from google.api_core.client_options import ClientOptions
from google.api_core import gapic_v1
from google.api_core import retry_async as retries
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore
import google.protobuf


try:
    OptionalRetry = Union[
        retries.AsyncRetry, gapic_v1.method._MethodDefault, None
    ]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.AsyncRetry, object, None]  # type: ignore

from google.ads.googleads.v19.services.types import customer_customizer_service
from google.rpc import status_pb2  # type: ignore
from .transports.base import (
    CustomerCustomizerServiceTransport,
    DEFAULT_CLIENT_INFO,
)
from .client import CustomerCustomizerServiceClient

try:
    from google.api_core import client_logging  # type: ignore

    CLIENT_LOGGING_SUPPORTED = True  # pragma: NO COVER
except ImportError:  # pragma: NO COVER
    CLIENT_LOGGING_SUPPORTED = False

_LOGGER = std_logging.getLogger(__name__)


class CustomerCustomizerServiceAsyncClient:
    """Service to manage customer customizer"""

    _client: CustomerCustomizerServiceClient

    # Copy defaults from the synchronous client for use here.
    # Note: DEFAULT_ENDPOINT is deprecated. Use _DEFAULT_ENDPOINT_TEMPLATE instead.
    DEFAULT_ENDPOINT = CustomerCustomizerServiceClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = (
        CustomerCustomizerServiceClient.DEFAULT_MTLS_ENDPOINT
    )
    _DEFAULT_ENDPOINT_TEMPLATE = (
        CustomerCustomizerServiceClient._DEFAULT_ENDPOINT_TEMPLATE
    )
    _DEFAULT_UNIVERSE = CustomerCustomizerServiceClient._DEFAULT_UNIVERSE

    customer_customizer_path = staticmethod(
        CustomerCustomizerServiceClient.customer_customizer_path
    )
    parse_customer_customizer_path = staticmethod(
        CustomerCustomizerServiceClient.parse_customer_customizer_path
    )
    customizer_attribute_path = staticmethod(
        CustomerCustomizerServiceClient.customizer_attribute_path
    )
    parse_customizer_attribute_path = staticmethod(
        CustomerCustomizerServiceClient.parse_customizer_attribute_path
    )
    common_billing_account_path = staticmethod(
        CustomerCustomizerServiceClient.common_billing_account_path
    )
    parse_common_billing_account_path = staticmethod(
        CustomerCustomizerServiceClient.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(
        CustomerCustomizerServiceClient.common_folder_path
    )
    parse_common_folder_path = staticmethod(
        CustomerCustomizerServiceClient.parse_common_folder_path
    )
    common_organization_path = staticmethod(
        CustomerCustomizerServiceClient.common_organization_path
    )
    parse_common_organization_path = staticmethod(
        CustomerCustomizerServiceClient.parse_common_organization_path
    )
    common_project_path = staticmethod(
        CustomerCustomizerServiceClient.common_project_path
    )
    parse_common_project_path = staticmethod(
        CustomerCustomizerServiceClient.parse_common_project_path
    )
    common_location_path = staticmethod(
        CustomerCustomizerServiceClient.common_location_path
    )
    parse_common_location_path = staticmethod(
        CustomerCustomizerServiceClient.parse_common_location_path
    )

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            CustomerCustomizerServiceAsyncClient: The constructed client.
        """
        return CustomerCustomizerServiceClient.from_service_account_info.__func__(CustomerCustomizerServiceAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            CustomerCustomizerServiceAsyncClient: The constructed client.
        """
        return CustomerCustomizerServiceClient.from_service_account_file.__func__(CustomerCustomizerServiceAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: Optional[ClientOptions] = None
    ):
        """Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variable is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """
        return CustomerCustomizerServiceClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore

    @property
    def transport(self) -> CustomerCustomizerServiceTransport:
        """Returns the transport used by the client instance.

        Returns:
            CustomerCustomizerServiceTransport: The transport used by the client instance.
        """
        return self._client.transport

    @property
    def api_endpoint(self):
        """Return the API endpoint used by the client instance.

        Returns:
            str: The API endpoint used by the client instance.
        """
        return self._client._api_endpoint

    @property
    def universe_domain(self) -> str:
        """Return the universe domain used by the client instance.

        Returns:
            str: The universe domain used
                by the client instance.
        """
        return self._client._universe_domain

    get_transport_class = CustomerCustomizerServiceClient.get_transport_class

    def __init__(
        self,
        *,
        credentials: Optional[ga_credentials.Credentials] = None,
        transport: Optional[
            Union[
                str,
                CustomerCustomizerServiceTransport,
                Callable[..., CustomerCustomizerServiceTransport],
            ]
        ] = "grpc_asyncio",
        client_options: Optional[ClientOptions] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the customer customizer service async client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Optional[Union[str,CustomerCustomizerServiceTransport,Callable[..., CustomerCustomizerServiceTransport]]]):
                The transport to use, or a Callable that constructs and returns a new transport to use.
                If a Callable is given, it will be called with the same set of initialization
                arguments as used in the CustomerCustomizerServiceTransport constructor.
                If set to None, a transport is chosen automatically.
            client_options (Optional[Union[google.api_core.client_options.ClientOptions, dict]]):
                Custom options for the client.

                1. The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client when ``transport`` is
                not explicitly provided. Only if this property is not set and
                ``transport`` was not explicitly provided, the endpoint is
                determined by the GOOGLE_API_USE_MTLS_ENDPOINT environment
                variable, which have one of the following values:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto-switch to the
                default mTLS endpoint if client certificate is present; this is
                the default value).

                2. If the GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide a client certificate for mTLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

                3. The ``universe_domain`` property can be used to override the
                default "googleapis.com" universe. Note that ``api_endpoint``
                property still takes precedence; and ``universe_domain`` is
                currently not supported for mTLS.

            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = CustomerCustomizerServiceClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

        if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
            std_logging.DEBUG
        ):  # pragma: NO COVER
            _LOGGER.debug(
                "Created client `google.ads.googleads.v19.services.CustomerCustomizerServiceAsyncClient`.",
                extra=(
                    {
                        "serviceName": "google.ads.googleads.v19.services.CustomerCustomizerService",
                        "universeDomain": getattr(
                            self._client._transport._credentials,
                            "universe_domain",
                            "",
                        ),
                        "credentialsType": f"{type(self._client._transport._credentials).__module__}.{type(self._client._transport._credentials).__qualname__}",
                        "credentialsInfo": getattr(
                            self.transport._credentials,
                            "get_cred_info",
                            lambda: None,
                        )(),
                    }
                    if hasattr(self._client._transport, "_credentials")
                    else {
                        "serviceName": "google.ads.googleads.v19.services.CustomerCustomizerService",
                        "credentialsType": None,
                    }
                ),
            )

    async def mutate_customer_customizers(
        self,
        request: Optional[
            Union[
                customer_customizer_service.MutateCustomerCustomizersRequest,
                dict,
            ]
        ] = None,
        *,
        customer_id: Optional[str] = None,
        operations: Optional[
            MutableSequence[
                customer_customizer_service.CustomerCustomizerOperation
            ]
        ] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> customer_customizer_service.MutateCustomerCustomizersResponse:
        r"""Creates, updates or removes customer customizers.
        Operation statuses are returned.

        Args:
            request (Optional[Union[google.ads.googleads.v19.services.types.MutateCustomerCustomizersRequest, dict]]):
                The request object. Request message for
                [CustomerCustomizerService.MutateCustomerCustomizers][google.ads.googleads.v19.services.CustomerCustomizerService.MutateCustomerCustomizers].
            customer_id (:class:`str`):
                Required. The ID of the customer
                whose customer customizers are being
                modified.

                This corresponds to the ``customer_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            operations (:class:`MutableSequence[google.ads.googleads.v19.services.types.CustomerCustomizerOperation]`):
                Required. The list of operations to
                perform on individual customer
                customizers.

                This corresponds to the ``operations`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.ads.googleads.v19.services.types.MutateCustomerCustomizersResponse:
                Response message for a customizer
                attribute mutate.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [customer_id, operations]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(
            request,
            customer_customizer_service.MutateCustomerCustomizersRequest,
        ):
            request = (
                customer_customizer_service.MutateCustomerCustomizersRequest(
                    request
                )
            )

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if customer_id is not None:
            request.customer_id = customer_id
        if operations:
            request.operations.extend(operations)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.mutate_customer_customizers
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("customer_id", request.customer_id),)
            ),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def __aenter__(self) -> "CustomerCustomizerServiceAsyncClient":
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.transport.close()


DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=package_version.__version__
)

if hasattr(DEFAULT_CLIENT_INFO, "protobuf_runtime_version"):  # pragma: NO COVER
    DEFAULT_CLIENT_INFO.protobuf_runtime_version = google.protobuf.__version__


__all__ = ("CustomerCustomizerServiceAsyncClient",)
